@@ -14,9 +14,11 @@ if not modules then modules = { } end modules ['font-syn'] = {
 -- old ff  loader: 140 sec
 -- new lua loader:   5 sec
 
+-- maybe find(...,strictname,1,true)
+
 local next, tonumber, type, tostring = next, tonumber, type, tostring
 local sub, gsub, match, find, lower, upper = string.sub, string.gsub, string.match, string.find, string.lower, string.upper
-local concat, sort, fastcopy = table.concat, table.sort, table.fastcopy
+local concat, sort, fastcopy, tohash = table.concat, table.sort, table.fastcopy, table.tohash
 local serialize, sortedhash = table.serialize, table.sortedhash
 local lpegmatch = lpeg.match
 local unpack = unpack or table.unpack
@@ -24,6 +26,7 @@ local formatters, topattern = string.formatters, string.topattern
 local round = math.round
 local P, R, S, C, Cc, Ct, Cs = lpeg.P, lpeg.R, lpeg.S, lpeg.C, lpeg.Cc, lpeg.Ct, lpeg.Cs
 local lpegmatch, lpegpatterns = lpeg.match, lpeg.patterns
+local isfile, modificationtime = lfs.isfile, lfs.modification
 
 local allocate             = utilities.storage.allocate
 local sparse               = utilities.storage.sparse
@@ -35,6 +38,7 @@ local splitname            = file.splitname
 local basename             = file.basename
 local nameonly             = file.nameonly
 local pathpart             = file.pathpart
+local suffixonly           = file.suffix
 local filejoin             = file.join
 local is_qualified_path    = file.is_qualified_path
 local exists               = io.exists
@@ -43,9 +47,6 @@ local findfile             = resolvers.findfile
 local cleanpath            = resolvers.cleanpath
 local resolveprefix        = resolvers.resolve
 
------ fontloader           = fontloader -- still needed for pfb (now)
------ get_font_info        = fontloader.info
-
 local settings_to_hash     = utilities.parsers.settings_to_hash_tolerant
 
 local trace_names          = false  trackers.register("fonts.names",          function(v) trace_names          = v end)
@@ -55,10 +56,8 @@ local trace_rejections     = false  trackers.register("fonts.rejections",     fu
 
 local report_names         = logs.reporter("fonts","names")
 
---[[ldx--
-<p>This module implements a name to filename resolver. Names are resolved
-using a table that has keys filtered from the font related files.</p>
---ldx]]--
+-- This module implements a name to filename resolver. Names are resolved using a
+-- table that has keys filtered from the font related files.
 
 fonts                      = fonts or { } -- also used elsewhere
 
@@ -87,10 +86,6 @@ local autoreload           = true
 directives.register("fonts.autoreload",     function(v) autoreload     = toboolean(v) end)
 directives.register("fonts.usesystemfonts", function(v) usesystemfonts = toboolean(v) end)
 
---[[ldx--
-<p>A few helpers.</p>
---ldx]]--
-
 -- -- what to do with these -- --
 --
 -- thin -> thin
@@ -304,10 +299,8 @@ local function analyzespec(somename)
     end
 end
 
---[[ldx--
-<p>It would make sense to implement the filters in the related modules,
-but to keep the overview, we define them here.</p>
---ldx]]--
+-- It would make sense to implement the filters in the related modules, but to keep
+-- the overview, we define them here.
 
 filters.afm = fonts.handlers.afm.readers.getinfo
 filters.otf = fonts.handlers.otf.readers.getinfo
@@ -393,11 +386,11 @@ filters.ttc = filters.otf
 --         local hash = { }
 --         local okay = false
 --         for line in f:lines() do -- slow but only a few lines at the beginning
---             if find(line,"dict begin") then
+--             if find(line,"dict begin",1,true) then
 --                 okay = true
 --             elseif not okay then
 --                 -- go on
---             elseif find(line,"currentdict end") then
+--             elseif find(line,"currentdict end",1,true) then
 --                 break
 --             else
 --                 local key, value = lpegmatch(p_entry,line)
@@ -411,11 +404,9 @@ filters.ttc = filters.otf
 --     end
 -- end
 
---[[ldx--
-<p>The scanner loops over the filters using the information stored in
-the file databases. Watch how we check not only for the names, but also
-for combination with the weight of a font.</p>
---ldx]]--
+-- The scanner loops over the filters using the information stored in the file
+-- databases. Watch how we check not only for the names, but also for combination
+-- with the weight of a font.
 
 filters.list = {
     "otf", "ttf", "ttc", "afm", -- no longer dfont support (for now)
@@ -423,8 +414,10 @@ filters.list = {
 
 -- to be considered: loop over paths per list entry (so first all otf ttf etc)
 
-names.fontconfigfile    = "fonts.conf" -- a bit weird format, bonus feature
-names.osfontdirvariable = "OSFONTDIR"  -- the official way, in minimals etc
+names.fontconfigfile       = "fonts.conf"   -- a bit weird format, bonus feature
+names.osfontdirvariable    = "OSFONTDIR"    -- the official way, in minimals etc
+names.extrafontsvariable   = "EXTRAFONTS"   -- the official way, in minimals etc
+names.runtimefontsvariable = "RUNTIMEFONTS" -- the official way, in minimals etc
 
 filters.paths = { }
 filters.names = { }
@@ -436,7 +429,7 @@ function names.getpaths(trace)
             local v = cleanpath(t[i])
             v = gsub(v,"/+$","") -- not needed any more
             local key = lower(v)
-            report_names("%a specifies path %a",where,v)
+            report_names("variable %a specifies path %a",where,v)
             if not hash[key] then
                 r = r + 1
                 result[r] = v
@@ -448,6 +441,10 @@ function names.getpaths(trace)
     if path ~= "" then
         collect(resolvers.expandedpathlist(path),path)
     end
+    local path = names.extrafontsvariable or ""
+    if path ~= "" then
+        collect(resolvers.expandedpathlist(path),path)
+    end
     if xml then
         local confname = resolvers.expansion("FONTCONFIG_FILE") or ""
         if confname == "" then
@@ -459,11 +456,11 @@ function names.getpaths(trace)
             if name == "" then
                 -- after all, fontconfig is a unix thing
                 name = filejoin("/etc",confname)
-                if not lfs.isfile(name) then
+                if not isfile(name) then
                     name = "" -- force quit
                 end
             end
-            if name ~= "" and lfs.isfile(name) then
+            if name ~= "" and isfile(name) then
                 if trace_names then
                     report_names("%s fontconfig file %a","loading",name)
                 end
@@ -476,7 +473,7 @@ function names.getpaths(trace)
                             incname = filejoin(path,incname)
                         end
                     end
-                    if lfs.isfile(incname) then
+                    if isfile(incname) then
                         if trace_names then
                             report_names("%s fontconfig file %a","merging included",incname)
                         end
@@ -506,15 +503,20 @@ local function cleanname(name)
 end
 
 local function cleanfilename(fullname,defaultsuffix)
-    local path, name, suffix = splitname(fullname)
-    name = gsub(lower(name),"[^%a%d]","")
-    if suffix and suffix ~= "" then
-        return name .. ".".. suffix
-    elseif defaultsuffix and defaultsuffix ~= "" then
-        return name .. ".".. defaultsuffix
-    else
-        return name
+    if fullname then
+        local path, name, suffix = splitname(fullname)
+        if name then
+            name = gsub(lower(name),"[^%a%d]","")
+            if suffix and suffix ~= "" then
+                return name .. ".".. suffix
+            elseif defaultsuffix and defaultsuffix ~= "" then
+                return name .. ".".. defaultsuffix
+            else
+                return name
+            end
+        end
     end
+    return "badfontname"
 end
 
 local sorter = function(a,b)
@@ -539,23 +541,6 @@ names.cleanfilename = cleanfilename
 --     return result
 -- end
 
-local function walk_tree(pathlist,suffix,identify)
-    if pathlist then
-        for i=1,#pathlist do
-            local path = pathlist[i]
-            path = cleanpath(path .. "/")
-            path = gsub(path,"/+","/")
-            local pattern = path .. "**." .. suffix -- ** forces recurse
-            report_names("globbing path %a",pattern)
-            local t = dir.glob(pattern)
-            sort(t,sorter)
-            for j=1,#t do
-                local completename = t[j]
-                identify(completename,basename(completename),suffix,completename)
-            end
-        end
-    end
-end
 
 local function check_name(data,result,filename,modification,suffix,subfont)
     -- shortcuts
@@ -1002,9 +987,11 @@ local function unpackreferences()
 end
 
 local function analyzefiles(olddata)
+
     if not trace_warnings then
         report_names("warnings are disabled (tracker 'fonts.warnings')")
     end
+
     local data               = names.data
     local done               = { }
     local totalnofread       = 0
@@ -1020,6 +1007,26 @@ local function analyzefiles(olddata)
     local oldspecifications  = olddata and olddata.specifications or { }
     local oldrejected        = olddata and olddata.rejected       or { }
     local treatmentdata      = treatments.data or { } -- when used outside context
+    ----- walked             = setmetatableindex("number")
+
+    local function walk_tree(pathlist,suffix,identify)
+        if pathlist then
+            for i=1,#pathlist do
+                local path = pathlist[i]
+                path = cleanpath(path .. "/")
+                path = gsub(path,"/+","/")
+                local pattern = path .. "**." .. suffix -- ** forces recurse
+                report_names("globbing path %a",pattern)
+                local t = dir.glob(pattern)
+                sort(t,sorter)
+                for j=1,#t do
+                    local completename = t[j]
+                    identify(completename,basename(completename),suffix,completename)
+                end
+             -- walked[path] = walked[path] + #t
+            end
+        end
+    end
 
     local function identify(completename,name,suffix,storedname)
         local pathpart, basepart = splitbase(completename)
@@ -1080,7 +1087,7 @@ local function analyzefiles(olddata)
             end
             -- needs checking with ttc / ttx : date not updated ?
             local result = nil
-            local modification = lfs.attributes(completename,"modification")
+            local modification = modificationtime(completename)
             if olddata and modification and modification > 0 then
                 local oldindex = oldindices[storedname] -- index into specifications
                 if oldindex then
@@ -1123,6 +1130,7 @@ local function analyzefiles(olddata)
         end
         logs.flush() --  a bit overkill for each font, maybe not needed here
     end
+
     local function traverse(what, method)
         local list = filters.list
         for n=1,#list do
@@ -1141,7 +1149,9 @@ local function analyzefiles(olddata)
         end
         logs.flush()
     end
+
     -- problem .. this will not take care of duplicates
+
     local function withtree(suffix)
         resolvers.dowithfilesintree(".*%." .. suffix .. "$", function(method,root,path,name)
             if method == "file" or method == "tree" then
@@ -1155,19 +1165,23 @@ local function analyzefiles(olddata)
             report_names("scanning path %a for %s files",blobpath,suffix)
         end, function(blobtype,blobpath,pattern,total,checked,done)
             blobpath = resolveprefix(blobpath) -- no shortcut
-            report_names("%s entries found, %s %s files checked, %s okay",total,checked,suffix,done)
+            report_names("%s %s files checked, %s okay",checked,suffix,done)
         end)
     end
+
     local function withlsr(suffix) -- all trees
         -- we do this only for a stupid names run, not used for context itself,
         -- using the vars is too clumsy so we just stick to a full scan instead
         local pathlist = resolvers.splitpath(resolvers.showpath("ls-R") or "")
         walk_tree(pathlist,suffix,identify)
     end
+
     local function withsystem(suffix) -- OSFONTDIR cum suis
         walk_tree(names.getpaths(trace),suffix,identify)
     end
+
     traverse("tree",withtree) -- TEXTREE only
+
     if not usesystemfonts then
         report_names("ignoring system fonts")
     elseif texconfig.kpse_init then
@@ -1175,9 +1189,15 @@ local function analyzefiles(olddata)
     else
         traverse("system", withsystem)
     end
+
     data.statistics.readfiles      = totalnofread
     data.statistics.skippedfiles   = totalnofskipped
     data.statistics.duplicatefiles = totalnofduplicates
+
+ -- for k, v in sortedhash(walked) do
+ --     report_names("%s : %i",k,v)
+ -- end
+
 end
 
 local function addfilenames()
@@ -1372,20 +1392,34 @@ local function is_reloaded()
     end
 end
 
---[[ldx--
-<p>The resolver also checks if the cached names are loaded. Being clever
-here is for testing purposes only (it deals with names prefixed by an
-encoding name).</p>
---ldx]]--
+-- The resolver also checks if the cached names are loaded. Being clever here is for
+-- testing purposes only (it deals with names prefixed by an encoding name).
 
-local function fuzzy(mapping,sorted,name,sub)
+local function fuzzy(mapping,sorted,name,sub) -- no need for reverse sorted here
     local condensed = gsub(name,"[^%a%d]","")
+    local pattern   = condensed .. "$"
+    local matches   = false
     for k=1,#sorted do
         local v = sorted[k]
-        if find(v,condensed) then
+        if v == condensed then
             return mapping[v], v
+        elseif find(v,pattern) then
+            return mapping[v], v
+        elseif find(v,condensed) then
+            if matches then
+                matches[#matches+1] = v
+            else
+                matches = { v }
+            end
         end
     end
+    if matches then
+        if #matches > 1 then
+            sort(matches,function(a,b) return #a < #b end)
+        end
+        matches = matches[1]
+        return mapping[matches], matches
+    end
 end
 
 -- we could cache a lookup .. maybe some day ... (only when auto loaded!)
@@ -1492,10 +1526,54 @@ end
 --     end
 -- end
 
+local runtimefiles = { }
+local runtimedone  = false
+
+local function addruntimepath(path)
+    names.load()
+    local paths    = type(path) == "table" and path or { path }
+    local suffixes = tohash(filters.list)
+    for i=1,#paths do
+        local path = resolveprefix(paths[i])
+        if path ~= "" then
+            local list = dir.glob(path.."/*")
+            for i=1,#list do
+                local fullname = list[i]
+                local suffix   = lower(suffixonly(fullname))
+                if suffixes[suffix] then
+                    local c = cleanfilename(fullname)
+                    runtimefiles[c] = fullname
+                    if trace_names then
+                        report_names("adding runtime filename %a for %a",c,fullname)
+                    end
+                end
+            end
+        end
+    end
+end
+
+local function addruntimefiles(variable)
+    local paths = variable and resolvers.expandedpathlistfromvariable(variable)
+    if paths and #paths > 0 then
+        addruntimepath(paths)
+    end
+end
+
+names.addruntimepath  = addruntimepath
+names.addruntimefiles = addruntimefiles
+
 function names.getfilename(askedname,suffix) -- last resort, strip funny chars
+    if not runtimedone then
+        addruntimefiles(names.runtimefontsvariable)
+        runtimedone = true
+    end
+    local cleanname = cleanfilename(askedname,suffix)
+    local found     = runtimefiles[cleanname]
+    if found then
+        return found
+    end
     names.load()
     local files = names.data.files
-    local cleanname = cleanfilename(askedname,suffix)
     local found = files and files[cleanname] or ""
     if found == "" and is_reloaded() then
         files = names.data.files
@@ -1938,7 +2016,8 @@ local lastlookups, lastpattern = { }, ""
 
 local function look_them_up(lookups,specification)
     for key, value in sortedhash(specification) do
-        local t, n = { }, 0
+        local t = { }
+        local n = 0
         if find(value,"*",1,true) then
             value = topattern(value)
             for i=1,#lookups do
