@@ -14,7 +14,7 @@ local concat, insert, remove, sortedkeys, reversed = table.concat, table.insert,
 local format, sub, gsub, find, gmatch, match = string.format, string.sub, string.gsub, string.find, string.gmatch, string.match
 local type, next, tonumber, tostring, select = type, next, tonumber, tostring, select
 local lpegmatch = lpeg.match
-local P, S, C, Cc, Cs = lpeg.P, lpeg.S, lpeg.C, lpeg.Cc, lpeg.Cs
+local P, S, C = lpeg.P, lpeg.S, lpeg.C
 local patterns = lpeg.patterns
 local setmetatableindex = table.setmetatableindex
 local formatters, strip = string.formatters, string.strip
@@ -92,7 +92,7 @@ local forceraw           = false
 
 local p_texescape        = patterns.texescape
 
-local tokenizedcs        = context.tokenizedcs
+local tokenizedxmlw      = context.tokenizedcs and context.tokenizedcs.xmlw
 
 directives.enable("xml.path.keeplastmatch")
 
@@ -337,7 +337,7 @@ do
             noferrors = noferrors + 1
             errors[filename] = errors[filename] + 1
         end
-        errorhandler(filename)
+        errorhandler(message) -- (filename)
     end
 
     logs.registerfinalactions(function()
@@ -472,6 +472,7 @@ lxml.addindex = addindex
 implement {
     name      = "xmladdindex",
     arguments = "string",
+    public    = true,
     actions   = addindex,
 }
 
@@ -501,15 +502,14 @@ function lxml.checkindex(name)
     return root and root.index or 0
 end
 
-if tokenizedcs then
+if tokenizedxmlw then
 
     function lxml.withindex(name,n,command) -- will change as name is always there now
         local i, p = lpegmatch(splitter,n)
-        local w = tokenizedcs.xmlw
         if p then
-            contextsprint(ctxcatcodes,w,"{",command,"}{",n,"}")
+            contextsprint(ctxcatcodes,tokenizedxmlw,"{",command,"}{",n,"}")
         else
-            contextsprint(ctxcatcodes,w,"{",command,"}{",name,"::",n,"}")
+            contextsprint(ctxcatcodes,tokenizedxmlw,"{",command,"}{",name,"::",n,"}")
         end
     end
 
@@ -770,7 +770,9 @@ end)
 
 local tex_element
 
-if tokenizedcs then
+if tokenizedxmlw then
+
+-- local expandmacro = token.expandmacro
 
     tex_element = function(e,handlers)
         if setfilename then
@@ -788,16 +790,19 @@ if tokenizedcs then
             local tc = type(command)
             if tc == "string" then
                 local rootname, ix = e.name, e.ix
-                local w = tokenizedcs.xmlw
                 if rootname then
                     if not ix then
                         addindex(rootname,false,true)
                         ix = e.ix
                     end
-                    contextsprint(ctxcatcodes,w,"{",command,"}{",rootname,"::",ix,"}")
+-- lmtx only, same performance, a bit more immediate:
+--
+-- expandmacro(tokenizedxmlw,ctxcatcodes,true,command,true,rootname.."::"..ix)
+--
+                    contextsprint(ctxcatcodes,tokenizedxmlw,"{",command,"}{",rootname,"::",ix,"}")
                 else
                     report_lxml("fatal error: no index for %a",command)
-                    contextsprint(ctxcatcodes,w,"{",command,"}{",ix or 0,"}")
+                    contextsprint(ctxcatcodes,tokenizedxmlw,"{",command,"}{",ix or 0,"}")
                 end
             elseif tc == "function" then
                 command(e)
@@ -1488,11 +1493,10 @@ end
 
 local command
 
-if tokenizedcs then
+if tokenizedxmlw then
 
     command = function(collected,cmd,otherwise)
         local n = collected and #collected
-        local w = tokenizedcs.xmlw
         if n and n > 0 then
             local wildcard = find(cmd,"*",1,true)
             for c=1,n do -- maybe optimize for n=1
@@ -1506,13 +1510,13 @@ if tokenizedcs then
                 if not ix or not name then
                     report_lxml("no valid node index for element %a using command %s",name or "?",cmd)
                 elseif wildcard then
-                    contextsprint(ctxcatcodes,w,"{",(gsub(cmd,"%*",e.tg)),"}{",name,"::",ix,"}")
+                    contextsprint(ctxcatcodes,tokenizedxmlw,"{",(gsub(cmd,"%*",e.tg)),"}{",name,"::",ix,"}")
                 else
-                    contextsprint(ctxcatcodes,w,"{",cmd,"}{",name,"::",ix,"}")
+                    contextsprint(ctxcatcodes,tokenizedxmlw,"{",cmd,"}{",name,"::",ix,"}")
                 end
             end
         elseif otherwise then
-            contextsprint(ctxcatcodes,w,"{",otherwise,"}{#1}")
+            contextsprint(ctxcatcodes,tokenizedxmlw,"{",otherwise,"}{#1}")
         end
     end
 
@@ -1757,6 +1761,23 @@ local function concatlist(collected,separator,lastseparator,textonly) -- test th
     concatrange(collected,false,false,separator,lastseparator,textonly)
 end
 
+local function depth(collected)
+    local d = 0
+    if collected then
+        local c = collected and collected[1]
+        if c.tg then
+            while c do
+                d = d + 1
+                c = c.__p__
+                if not c then
+                    break
+                end
+            end
+        end
+    end
+    contextsprint(ctxcatcodes,d)
+end
+
 texfinalizers.first          = first
 texfinalizers.last           = last
 texfinalizers.all            = all
@@ -1780,6 +1801,7 @@ texfinalizers.concatrange    = concatrange
 texfinalizers.chainattribute = chainattribute
 texfinalizers.chainpath      = chainpath
 texfinalizers.default        = all -- !!
+texfinalizers.depth          = depth
 
 function texfinalizers.tag(collected,n)
     if collected then
@@ -1865,6 +1887,10 @@ lxml.verbatim = verbatim
 
 -- helpers
 
+function lxml.depth(id)
+    depth { getid(id) }
+end
+
 function lxml.first(id,pattern)
     local collected = xmlapplylpath(getid(id),pattern)
     if collected then
@@ -2044,6 +2070,28 @@ do
         end
     end
 
+    function lxml.ifatt(id,a,value)
+        local e = getid(id)
+        if e then
+            local at = e.at
+            att = at and at[a] or ""
+        else
+            att = ""
+        end
+        return att == value
+    end
+
+    function lxml.ifattempty(id,a)
+        local e = getid(id)
+        if e then
+            local at = e.at
+            att = at and at[a] or ""
+        else
+            att = ""
+        end
+        return att == ""
+    end
+
     function lxml.refatt(id,a)
         local e = getid(id)
         if e then
@@ -2073,7 +2121,8 @@ do
     implement {
         name      = "xmldoifatt",
         arguments = "3 strings",
-        actions = function(id,k,v)
+        public    = true,
+        actions   = function(id,k,v)
             local e = getid(id)
             ctx_doif(e and e.at[k] == v or false)
         end
@@ -2082,7 +2131,8 @@ do
     implement {
         name      = "xmldoifnotatt",
         arguments = "3 strings",
-        actions = function(id,k,v)
+        public    = true,
+        actions   = function(id,k,v)
             local e = getid(id)
             ctx_doifnot(e and e.at[k] == v or false)
         end
@@ -2091,7 +2141,8 @@ do
     implement {
         name      = "xmldoifelseatt",
         arguments = "3 strings",
-        actions = function(id,k,v)
+        public    = true,
+        actions   = function(id,k,v)
             local e = getid(id)
             ctx_doifelse(e and e.at[k] == v or false)
         end
@@ -2200,7 +2251,7 @@ function lxml.snippet(id,i)
     if e then
         local dt = e.dt
         if dt then
-            local dti = dt[i]
+            local dti = dt[tonumber(i)] -- string in lxml
             if dti then
                 xmlsprint(dti,e)
             end
@@ -2215,16 +2266,15 @@ function lxml.direct(id)
     end
 end
 
-if tokenizedcs then
+if tokenizedxmlw then
 
     function lxml.command(id,pattern,cmd)
         local i, p = getid(id,true)
-        local collected = xmlapplylpath(getid(i),pattern)
+        local collected = xmlapplylpath(getid(i),pattern) -- again getid?
         if collected then
             local nc = #collected
             if nc > 0 then
                 local rootname = p or i.name
-                local w = tokenizedcs.xmlw
                 for c=1,nc do
                     local e = collected[c]
                     local ix = e.ix
@@ -2232,7 +2282,7 @@ if tokenizedcs then
                         addindex(rootname,false,true)
                         ix = e.ix
                     end
-                    contextsprint(ctxcatcodes,w,"{",cmd,"}{",rootname,"::",ix,"}")
+                    contextsprint(ctxcatcodes,tokenizedxmlw,"{",cmd,"}{",rootname,"::",ix,"}")
                 end
             end
         end
@@ -2242,7 +2292,7 @@ else
 
     function lxml.command(id,pattern,cmd)
         local i, p = getid(id,true)
-        local collected = xmlapplylpath(getid(i),pattern)
+        local collected = xmlapplylpath(getid(i),pattern) -- again getid?
         if collected then
             local nc = #collected
             if nc > 0 then
@@ -2300,6 +2350,8 @@ do
         end
     end
 
+    xml.checkedempty = checkedempty
+
     function lxml.doifempty    (id,pattern) doif    (checkedempty(id,pattern)) end
     function lxml.doifnotempty (id,pattern) doifnot (checkedempty(id,pattern)) end
     function lxml.doifelseempty(id,pattern) doifelse(checkedempty(id,pattern)) end
@@ -2360,8 +2412,8 @@ function lxml.nonspace(id,pattern) -- slow, todo loop
     xmltprint(xmlcollect(getid(id),pattern,true))
 end
 
-function lxml.strip(id,pattern,nolines,anywhere)
-    xml.strip(getid(id),pattern,nolines,anywhere)
+function lxml.strip(id,pattern,nolines,anywhere,everywhere)
+    xml.strip(getid(id),pattern,nolines,anywhere,everywhere)
 end
 
 function lxml.stripped(id,pattern,nolines)
@@ -2705,6 +2757,8 @@ do
 
 end
 
+-- hm, maybe to ini to, these implements
+
 implement {
     name      = "xmlsetinjectors",
     actions   = xml.setinjectors,
