@@ -31,14 +31,14 @@ local getsubtype        = nuts.getsubtype
 local getattr           = nuts.getattr
 local getattrlist       = nuts.getattrlist
 local setattr           = nuts.setattr
-local getcomponents     = nuts.getcomponents -- not really needed
+----- getcomponents     = nuts.getcomponents -- not really needed
 local getwidth          = nuts.getwidth
 
 local getnucleus        = nuts.getnucleus
 local getsub            = nuts.getsub
 local getsup            = nuts.getsup
 
-local set_attributes    = nuts.setattributes
+local setattributes     = nuts.setattributes
 
 local nextnode          = nuts.traversers.node
 
@@ -52,17 +52,22 @@ local subbox_code       = nodecodes.subbox         -- attr list
 local submlist_code     = nodecodes.submlist       -- attr list
 local mathchar_code     = nodecodes.mathchar       -- attr fam char
 local mathtextchar_code = nodecodes.mathtextchar   -- attr fam char
-local delim_code        = nodecodes.delim          -- attr small_fam small_char large_fam large_char
+local delimiter_code    = nodecodes.delimiter      -- attr small_fam small_char large_fam large_char
 local style_code        = nodecodes.style          -- attr style
 local choice_code       = nodecodes.choice         -- attr display text script scriptscript
 local fence_code        = nodecodes.fence          -- attr subtype
 
 local accentcodes       = nodes.accentcodes
+local fencecodes        = nodes.fencecodes
 
 local fixedtopaccent_code    = accentcodes.fixedtop
 local fixedbottomaccent_code = accentcodes.fixedbottom
 local fixedbothaccent_code   = accentcodes.fixedboth
 
+local leftfence_code    = fencecodes.left
+local middlefence_code  = fencecodes.middle
+local rightfence_code   = fencecodes.right
+
 local kerncodes         = nodes.kerncodes
 
 local fontkern_code     = kerncodes.fontkern
@@ -94,7 +99,6 @@ local chardata          = characters.data
 local getmathcodes      = tex.getmathcodes
 local mathcodes         = mathematics.codes
 local ordinary_mathcode = mathcodes.ordinary
-local variable_mathcode = mathcodes.variable
 
 local fromunicode16     = fonts.mappings.fromunicode16
 local fontcharacters    = fonts.hashes.characters
@@ -116,10 +120,10 @@ local function processsubsup(start)
          -- start_tagged("mrow")
             process(nucleus)
          -- stop_tagged()
-            start_tagged("mrow")
+            start_tagged("mrow", { subscript = true })
             process(sub)
             stop_tagged()
-            start_tagged("mrow")
+            start_tagged("mrow", { superscript = true })
             process(sup)
             stop_tagged()
             stop_tagged()
@@ -158,7 +162,7 @@ local fencesstack = { }
 
 -- local function getunicode(n) -- instead of getchar
 --     local char = getchar(n)
---  -- local font = font_of_family(getfield(n,"fam")) -- font_of_family
+--  -- local font = getfontoffamily(getfield(n,"fam"))
 --     local font = getfont(n)
 --     local data = fontcharacters[font][char]
 --     return data.unicode or char
@@ -204,7 +208,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
     local mtexttag = nil
     while start do
         local id = getid(start)
-     -- showtag(start,id,true)
+-- showtag(start,id,true)
         if id == glyph_code or id == disc_code then
             if not mtexttag then
                 mtexttag = start_tagged("mtext")
@@ -221,7 +225,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                 local char = getchar(start)
                 local code = getmathcodes(char)
                 local tag
-                if code == ordinary_mathcode or code == variable_mathcode then
+                if code == ordinary_mathcode then
                     local ch = chardata[char]
                     local mc = ch and ch.mathclass
                     if mc == "number" then
@@ -254,7 +258,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                 stop_tagged()
              -- showtag(start,id,false)
                 break
-            elseif id == delim_code then
+            elseif id == delimiter_code then
                 -- check for code
                 setattr(start,a_tagged,start_tagged("mo"))
                 stop_tagged()
@@ -265,7 +269,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
             elseif id == noad_code then
              -- setattr(start,a_tagged,tags.current())
                 processsubsup(start)
-            elseif id == dubbox_code or id == hlist_code or id == vlist_code then
+            elseif id == subbox_code or id == hlist_code or id == vlist_code then
                 -- keep an eye on subbox_code and see what ends up in there
                 local attr = getattr(start,a_tagged)
                 if not attr then
@@ -292,7 +296,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                                 -- empty list
                             elseif not attr then
                                 -- box comes from strange place
-                                set_attributes(list,a_tagged,text) -- only the first node ?
+                                setattributes(list,a_tagged,text) -- only the first node ?
                             else
                                 -- Beware, the first node in list is the actual list so we definitely
                                 -- need to nest. This approach is a hack, maybe I'll make a proper
@@ -341,10 +345,10 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                                             runner(getlist(n),depth+1)
                                         elseif id == glyph_code then
                                             -- this should not be needed
-                                            local components = getcomponents(n) -- unlikely set
-                                            if components then
-                                                runner(getcomponent,depth+1)
-                                            end
+                                         -- local components = getcomponents(n) -- unlikely set
+                                         -- if components then
+                                         --     runner(getcomponent,depth+1)
+                                         -- end
                                         elseif id == disc_code then
                                             -- this should not be needed
                                             local pre, post, replace = getdisc(n)
@@ -372,7 +376,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                         end
                     end
                 end
-            elseif id == submlistcode then -- normally a hbox
+            elseif id == submlist_code then -- normally a hbox
                 local list = getlist(start)
                 if list then
                     local attr = getattr(start,a_tagged)
@@ -450,16 +454,16 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                     process(scriptscript)
                 end
             elseif id == fence_code then
+                local subtype = getsubtype(start)
                 local delim   = getfield(start,"delim")
-                local subtype = getfield(start,"subtype")
-                if subtype == 1 then
+                if subtype == leftfence_code then
                     -- left
                     local properties = { }
                     insert(fencesstack,properties)
                     setattr(start,a_tagged,start_tagged("mfenced",properties)) -- needs checking
                     if delim then
                         start_tagged("ignore")
-                        local chr = getfield(delim,"small_char")
+                        local chr = getchar(delim)
                         if chr ~= 0 then
                             properties.left = chr
                         end
@@ -467,12 +471,12 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                         stop_tagged()
                     end
                     start_tagged("mrow") -- begin of subsequence
-                elseif subtype == 2 then
+                elseif subtype == middlefence_code then
                     -- middle
                     if delim then
                         start_tagged("ignore")
                         local top = fencesstack[#fencesstack]
-                        local chr = getfield(delim,"small_char")
+                        local chr = getchar(delim)
                         if chr ~= 0 then
                             local mid = top.middle
                             if mid then
@@ -486,7 +490,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                     end
                     stop_tagged()        -- end of subsequence
                     start_tagged("mrow") -- begin of subsequence
-                elseif subtype == 3 then
+                elseif subtype == rightfence_code then
                     local properties = remove(fencesstack)
                     if not properties then
                         report_tags("missing right fence")
@@ -494,7 +498,7 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                     end
                     if delim then
                         start_tagged("ignore")
-                        local chr = getfield(delim,"small_char")
+                        local chr = getchar(delim)
                         if chr ~= 0 then
                             properties.right = chr
                         end
@@ -525,9 +529,9 @@ process = function(start) -- we cannot use the processor as we have no finalizer
                     stop_tagged()
                 end
             elseif id == accent_code then
+                local subtype    = getsubtype(start)
                 local accent     = getfield(start,"accent")
                 local bot_accent = getfield(start,"bot_accent")
-                local subtype    = getsubtype(start)
                 if bot_accent then
                     if accent then
                         setattr(start,a_tagged,start_tagged("munderover", {
@@ -582,9 +586,10 @@ end
 
 function noads.handlers.tags(head,style,penalties)
     start_tagged("math", { mode = (getattr(head,a_mathmode) == 1) and "display" or "inline" })
---     start_tagged("mrow")
     setattr(head,a_tagged,start_tagged("mrow"))
+-- showtag(head,getid(head),true)
     process(head)
+-- showtag(head,getid(head),false)
     stop_tagged()
     stop_tagged()
 end
