@@ -12,15 +12,18 @@ return {
     ["constants"] = {
         --
         "zerocount", "minusone", "minustwo", "plusone", "plustwo", "plusthree", "plusfour", "plusfive",
-        "plussix", "plusseven", "pluseight", "plusnine", "plusten", "plussixteen",
+        "plussix", "plusseven", "pluseight", "plusnine", "plusten", "pluseleven", "plustwelve", "plussixteen",
         "plusfifty", "plushundred", "plusonehundred", "plustwohundred", "plusfivehundred",
-        "plusthousand", "plustenthousand", "plustwentythousand", "medcard", "maxcard", "maxcardminusone",
+        "plusthousand", "plustenthousand", "plustwentythousand", "medcard", "maxcard", "maxcardminusone", "maxiterator",
         "zeropoint", "onepoint", "halfapoint", "onebasepoint", "maxcount", "maxdimen", "scaledpoint", "thousandpoint",
         "points", "halfpoint",
-        "zeroskip",
+        "zeroskip", "centeringskip", "stretchingskip", "shrinkingskip",
+        "centeringfillskip", "stretchingfillskip", "shrinkingfillskip",
         "zeromuskip", "onemuskip",
         "pluscxxvii", "pluscxxviii", "pluscclv", "pluscclvi",
         "normalpagebox",
+        "binaryshiftedten", "binaryshiftedtwenty", "binaryshiftedthirty",
+        "thickermuskip",
         --
         "directionlefttoright", "directionrighttoleft",
         --
@@ -28,13 +31,15 @@ return {
         --
         "emptytoks", "empty", "undefined",
         --
+        "prerollrun",
+        --
         "voidbox", "emptybox", "emptyvbox", "emptyhbox",
         --
         "bigskipamount", "medskipamount", "smallskipamount",
         --
         "fmtname", "fmtversion", "texengine", "texenginename", "texengineversion", "texenginefunctionality",
         "luatexengine", "pdftexengine", "xetexengine", "unknownengine",
-        "contextformat", "contextversion", "contextkind", "contextlmtxmode", "contextmark", "mksuffix",
+        "contextformat", "contextversion", "contextlmtxmode", "contextmark", "mksuffix",
         --
         "activecatcode",
         --
@@ -45,26 +50,37 @@ return {
         --
         "attributeunsetvalue",
         --
+        "statuswrite",
+        --
         "uprotationangle", "rightrotationangle", "downrotationangle", "leftrotationangle",
         --
         "inicatcodes",
         "ctxcatcodes", "texcatcodes", "notcatcodes", "txtcatcodes", "vrbcatcodes",
         "prtcatcodes", "nilcatcodes", "luacatcodes", "tpacatcodes", "tpbcatcodes",
-        "xmlcatcodes", "ctdcatcodes",
+        "xmlcatcodes", "ctdcatcodes", "rlncatcodes",
         --
         "escapecatcode", "begingroupcatcode", "endgroupcatcode", "mathshiftcatcode", "alignmentcatcode",
         "endoflinecatcode", "parametercatcode", "superscriptcatcode", "subscriptcatcode", "ignorecatcode",
         "spacecatcode", "lettercatcode", "othercatcode", "activecatcode", "commentcatcode", "invalidcatcode",
         --
         "tabasciicode", "newlineasciicode", "formfeedasciicode", "endoflineasciicode", "endoffileasciicode",
-        "spaceasciicode", "hashasciicode", "dollarasciicode", "commentasciicode", "ampersandasciicode",
-        "colonasciicode", "backslashasciicode", "circumflexasciicode", "underscoreasciicode",
+        "commaasciicode", "spaceasciicode", "periodasciicode",
+        "hashasciicode", "dollarasciicode", "commentasciicode", "ampersandasciicode",
+        "colonasciicode", "semicolonasciicode", "backslashasciicode", "circumflexasciicode", "underscoreasciicode",
         "leftbraceasciicode", "barasciicode", "rightbraceasciicode", "tildeasciicode", "delasciicode",
         "leftparentasciicode", "rightparentasciicode",
         "lessthanasciicode", "morethanasciicode", "doublecommentsignal",
         "atsignasciicode", "exclamationmarkasciicode", "questionmarkasciicode",
         "doublequoteasciicode", "singlequoteasciicode", "forwardslashasciicode",
-        "primeasciicode", "hyphenasciicode",
+        "primeasciicode", "hyphenasciicode", "percentasciicode", "leftbracketasciicode", "rightbracketasciicode",
+        --
+        "hsizefrozenparcode", "skipfrozenparcode", "hangfrozenparcode", "indentfrozenparcode", "parfillfrozenparcode",
+        "adjustfrozenparcode", "protrudefrozenparcode", "tolerancefrozenparcode", "stretchfrozenparcode",
+        "loosenessfrozenparcode", "lastlinefrozenparcode", "linepenaltyfrozenparcode", "clubpenaltyfrozenparcode",
+        "widowpenaltyfrozenparcode", "displaypenaltyfrozenparcode", "brokenpenaltyfrozenparcode",
+        "demeritsfrozenparcode", "shapefrozenparcode", "linefrozenparcode", "hyphenationfrozenparcode",
+        "shapingpenaltyfrozenparcode", "orphanpenaltyfrozenparcode", "allfrozenparcode",
+        "mathpenaltyfrozenparcode",
         --
         "activemathcharcode",
         --
@@ -74,22 +90,52 @@ return {
         --
         "bottomlevelgroupcode", "simplegroupcode", "hboxgroupcode", "adjustedhboxgroupcode", "vboxgroupcode",
         "vtopgroupcode", "aligngroupcode", "noaligngroupcode", "outputgroupcode", "mathgroupcode",
-        "discretionarygroupcode", "insertgroupcode", "vcentergroupcode", "mathchoicegroupcode",
-        "semisimplegroupcode", "mathshiftgroupcode", "mathleftgroupcode", "vadjustgroupcode",
+        "discretionarygroupcode", "insertgroupcode", "vadjustgroupcode", "vcentergroupcode", "mathabovegroupcode",
+        "mathchoicegroupcode", "alsosimplegroupcode", "semisimplegroupcode", "mathshiftgroupcode", "mathleftgroupcode",
+        "localboxgroupcode", "splitoffgroupcode", "splitkeepgroupcode", "preamblegroupcode",
+        "alignsetgroupcode", "finrowgroupcode", "discretionarygroupcode",
+        --
+        "markautomigrationcode", "insertautomigrationcode", "adjustautomigrationcode", "preautomigrationcode", "postautomigrationcode",
         --
         "charnodecode", "hlistnodecode", "vlistnodecode", "rulenodecode", "insertnodecode", "marknodecode",
         "adjustnodecode", "ligaturenodecode", "discretionarynodecode", "whatsitnodecode", "mathnodecode",
         "gluenodecode", "kernnodecode", "penaltynodecode", "unsetnodecode", "mathsnodecode",
         --
-        "charifcode", "catifcode", "numifcode", "dimifcode", "oddifcode", "vmodeifcode", "hmodeifcode",
-        "mmodeifcode", "innerifcode", "voidifcode", "hboxifcode", "vboxifcode", "xifcode", "eofifcode",
-        "trueifcode", "falseifcode", "caseifcode", "definedifcode", "csnameifcode", "fontcharifcode",
+     -- "charifcode", "catifcode", "numifcode", "dimifcode", "oddifcode", "vmodeifcode", "hmodeifcode",
+     -- "mmodeifcode", "innerifcode", "voidifcode", "hboxifcode", "vboxifcode", "xifcode", "eofifcode",
+     -- "trueifcode", "falseifcode", "caseifcode", "definedifcode", "csnameifcode", "fontcharifcode",
+        --
+        "overrulemathcontrolcode", "underrulemathcontrolcode", "radicalrulemathcontrolcode", "fractionrulemathcontrolcode",
+        "accentskewhalfmathcontrolcode", "accentskewapplymathcontrolcode", "applyordinarykernpairmathcontrolcode",
+        "applyverticalitalickernmathcontrolcode", "applyordinaryitalickernmathcontrolcode", "applycharitalickernmathcontrolcode",
+        "reboxcharitalickernmathcontrolcode", "applyboxeditalickernmathcontrolcode", "staircasekernmathcontrolcode",
+        "applytextitalickernmathcontrolcode", "applyscriptitalickernmathcontrolcode",
+        "checkspaceitalickernmathcontrolcode", "checktextitalickernmathcontrolcode",
+        "analyzescriptnucleuscharmathcontrolcode", "analyzescriptnucleuslistmathcontrolcode", "analyzescriptnucleusboxmathcontrolcode",
+        "accenttopskewwithoffsetmathcontrolcode", "ignorekerndimensionsmathcontrolcode", "ignoreflataccentsmathcontrolcode",
+        "extendaccentsmathcontrolcode", "extenddelimitersmathcontrolcode",
+        --
+        "noligaturingglyphoptioncode", "nokerningglyphoptioncode", "noexpansionglyphoptioncode", "noprotrusionglyphoptioncode",
+        "noleftkerningglyphoptioncode", "noleftligaturingglyphoptioncode", "norightkerningglyphoptioncode", "norightligaturingglyphoptioncode",
+        "noitaliccorrectionglyphoptioncode", "islargeoperatorglyphoptioncode", "hasitalicshapeglyphoptioncode",
+        --
+        "normalparcontextcode", "vmodeparcontextcode", "vboxparcontextcode", "vtopparcontextcode", "vcenterparcontextcode",
+        "vadjustparcontextcode", "insertparcontextcode", "outputparcontextcode", "alignparcontextcode",
+        "noalignparcontextcode", "spanparcontextcode", "resetparcontextcode",
+        --
+        "leftoriginlistanchorcode", "leftheightlistanchorcode", "leftdepthlistanchorcode",
+        "rightoriginlistanchorcode", "rightheightlistanchorcode", "rightdepthlistanchorcode",
+        "centeroriginlistanchorcode", "centerheightlistanchorcode", "centerdepthlistanchorcode",
+        "halfwaytotallistanchorcode", "halfwayheightlistanchorcode", "halfwaydepthlistanchorcode",
+        "halfwayleftlistanchorcode", "halfwayrightlistanchorcode",
+        --
+        "negatexlistsigncode", "negateylistsigncode", "negatelistsigncode",
         --
         "fontslantperpoint", "fontinterwordspace", "fontinterwordstretch", "fontinterwordshrink",
         "fontexheight", "fontemwidth", "fontextraspace", "slantperpoint",
         "mathexheight", "mathemwidth",
         "interwordspace", "interwordstretch", "interwordshrink", "exheight", "emwidth", "extraspace",
-        "mathsupdisplay", "mathsupnormal", "mathsupcramped", "mathsubnormal", "mathsubcombined",  "mathaxisheight",
+        "mathaxisheight",
         "muquad",
         --
         -- maybe a different class
@@ -100,7 +146,7 @@ return {
         "startallmodes", "stopallmodes", "startnotallmodes", "stopnotallmodes",
         "doifallmodes", "doifelseallmodes", "doifallmodeselse", "doifnotallmodes",
         "startenvironment", "stopenvironment", "environment",
-        "startcomponent", "stopcomponent", "component",
+        "startcomponent", "stopcomponent", "component", "startlocalcomponent", "stoplocalcomponent",
         "startproduct", "stopproduct", "product",
         "startproject", "stopproject", "project",
         "starttext", "stoptext", "startnotext", "stopnotext",
@@ -118,14 +164,21 @@ return {
         --
         "typescriptone", "typescripttwo", "typescriptthree", "mathsizesuffix",
         --
-        "mathordcode", "mathopcode", "mathbincode", "mathrelcode", "mathopencode", "mathclosecode",
-        "mathpunctcode", "mathalphacode", "mathinnercode", "mathnothingcode", "mathlimopcode",
-        "mathnolopcode", "mathboxcode", "mathchoicecode", "mathaccentcode", "mathradicalcode",
+        "mathordinarycode", "mathordcode", "mathoperatorcode", "mathopcode", "mathbinarycode", "mathbincode",
+        "mathrelationcode", "mathrelcode", "mathopencode", "mathclosecode", "mathpunctuationcode",
+        "mathpunctcode", "mathovercode", "mathundercode", "mathinnercode", "mathradicalcode",
+        "mathfractioncode", "mathmiddlecode", "mathaccentcode", "mathfencedcode", "mathghostcode",
+        "mathvariablecode",  "mathactivecode", "mathvcentercode", "mathconstructcode", "mathwrappedcode",
+        "mathbegincode", "mathendcode", "mathexplicitcode", "mathdivisioncode", "mathfactorialcode",
+        "mathdimensioncode", "mathexperimentalcode", "mathtextpunctuationcode",
+        "mathimaginarycode", "mathdifferentialcode", "mathexponentialcode", "mathellipsiscode", "mathfunctioncode", "mathdigitcode",
+     -- "mathtopaccentcode", "mathbottomaccentcode", "mathdelimitercode", "mathrootcode", "mathprintcode",        --
+        "mathalphacode", "mathboxcode", "mathchoicecode", "mathnothingcode", "mathlimopcode", "mathnolopcode",
+        "mathunsetcode", "mathunspacedcode", "mathallcode", "mathfakecode", "mathunarycode",
+        "mathmaybeordinarycode", "mathmayberelationcode", "mathmaybebinarycode", "mathnumbergroupcode",
         --
         "constantnumber", "constantnumberargument", "constantdimen", "constantdimenargument", "constantemptyargument",
         --
-        "continueifinputfile",
-        --
         "luastringsep", "!!bs", "!!es",
         --
         "lefttorightmark", "righttoleftmark", "lrm", "rlm",
@@ -136,11 +189,52 @@ return {
         "twoperemspace", "threeperemspace", "fourperemspace", "fiveperemspace", "sixperemspace",
         "figurespace", "punctuationspace", "hairspace", "enquad", "emquad",
         "zerowidthspace", "zerowidthnonjoiner", "zerowidthjoiner", "zwnj", "zwj",
-        "optionalspace", "asciispacechar", "softhyphen",
+        "optionalspace", "asciispacechar", "softhyphen", "autoinsertedspace",
         --
-        "Ux", "eUx", "Umathaccents",
+        "Ux", "eUx",
         --
         "parfillleftskip", "parfillrightskip",
+        --
+        "startlmtxmode", "stoplmtxmode", "startmkivmode", "stopmkivmode",
+        --
+        "wildcardsymbol",
+        --
+        "normalhyphenationcode", "automatichyphenationcode", "explicithyphenationcode", "syllablehyphenationcode", "uppercasehyphenationcode",
+        "collapsehyphenationcode", "compoundhyphenationcode", "strictstarthyphenationcode", "strictendhyphenationcode",
+        "automaticpenaltyhyphenationcode", "explicitpenaltyhyphenationcode", "permitgluehyphenationcode", "permitallhyphenationcode",
+        "permitmathreplacehyphenationcode", "forcecheckhyphenationcode", "lazyligatureshyphenationcode", "forcehandlerhyphenationcode",
+        "feedbackcompoundhyphenationcode", "ignoreboundshyphenationcode", "partialhyphenationcode", "completehyphenationcode",
+        --
+        "normalizelinenormalizecode", "parindentskipnormalizecode", "swaphangindentnormalizecode", "swapparsshapenormalizecode",
+        "breakafterdirnormalizecode", "removemarginkernsnormalizecode", "clipwidthnormalizecode", "flattendiscretionariesnormalizecode",
+        "discardzerotabskipsnormalizecode", "flattenhleadersnormalizecode",
+        --
+        "normalizeparnormalizeparcode", "flattenvleadersnormalizeparcode",
+        --
+        "nopreslackclassoptioncode", "nopostslackclassoptioncode",
+        "lefttopkernclassoptioncode", "righttopkernclassoptioncode", "leftbottomkernclassoptioncode", "rightbottomkernclassoptioncode",
+        "lookaheadforendclassoptioncode", "noitaliccorrectionclassoptioncode",  "defaultmathclassoptions",
+     -- "openfenceclassoptioncode", "closefenceclassoptioncode", "middlefenceclassoptioncode",
+        "checkligatureclassoptioncode", "checkitaliccorrectionclassoptioncode", "checkkernpairclassoptioncode",
+        "flattenclassoptioncode", "omitpenaltyclassoptioncode", "unpackclassoptioncode", "raiseprimeclassoptioncode",
+        "carryoverlefttopkernclassoptioncode", "carryoverleftbottomkernclassoptioncode", "carryoverrighttopkernclassoptioncode", "carryoverrightbottomkernclassoptioncode",
+        "preferdelimiterdimensionsclassoptioncode", "autoinjectclassoptioncode", "removeitaliccorrectionclassoptioncode",
+        "operatoritaliccorrectionclassoptioncode",
+        --
+        "noligaturingglyphoptioncode", "nokerningglyphoptioncode", "noleftligatureglyphoptioncode",
+        "noleftkernglyphoptioncode", "norightligatureglyphoptioncode", "norightkernglyphoptioncode",
+        "noexpansionglyphoptioncode", "noprotrusionglyphoptioncode", "noitaliccorrectionglyphoptioncode",
+        -- extras:
+        "nokerningcode", "noligaturingcode",
+        --
+        "frozenflagcode", "tolerantflagcode", "protectedflagcode", "primitiveflagcode", "permanentflagcode", "noalignedflagcode", "immutableflagcode",
+        "mutableflagcode", "globalflagcode", "overloadedflagcode", "immediateflagcode", "conditionalflagcode", "valueflagcode", "instanceflagcode",
+        --
+        "ordmathflattencode", "binmathflattencode", "relmathflattencode", "punctmathflattencode", "innermathflattencode",
+        --
+        "normalworddiscoptioncode", "preworddiscoptioncode", "postworddiscoptioncode",
+        --
+        "continueifinputfile", "continuewhenlmtxmode", "continuewhenmkivmode",
     },
     ["helpers"] = {
         --
@@ -165,9 +259,12 @@ return {
         "globalsetmode", "globalresetmode", "globalsetsystemmode", "globalresetsystemmode",
         "booleanmodevalue",
         --
-        "newcount", "newdimen", "newskip", "newmuskip", "newbox", "newtoks", "newread", "newwrite", "newmarks", "newinsert", "newattribute", "newif",
+        "newcount", "newdimen", "newskip", "newmuskip", "newbox", "newtoks", "newread", "newwrite", "newmarks", "newinsert", "newattribute", "newif", "newfloat",
         "newlanguage", "newfamily", "newfam", "newhelp", -- not used
         --
+        "newinteger", "newdimension", "newgluespec", "newmugluespec", "newposit",
+        "aliasinteger", "aliasdimension", "aliasposit",
+        --
         "then",
         "begcsname",
         --
@@ -175,6 +272,7 @@ return {
         --
         "strippedcsname","checkedstrippedcsname",
         --
+        "nofarguments",
         "firstargumentfalse", "firstargumenttrue",
         "secondargumentfalse", "secondargumenttrue",
         "thirdargumentfalse", "thirdargumenttrue",
@@ -183,18 +281,20 @@ return {
         "sixthargumentfalse", "sixthargumenttrue",
         "seventhargumentfalse", "seventhargumenttrue",
         --
-        "vkern", "hkern",
+        "vkern", "hkern", "vpenalty", "hpenalty",
         --
         "doglobal", "dodoglobal", "redoglobal", "resetglobal",
         --
-        "donothing", "dontcomplain", "forgetall",
+        "donothing", "untraceddonothing", "dontcomplain", "moreboxtracing", "lessboxtracing", "noboxtracing", "forgetall",
         --
         "donetrue", "donefalse", "foundtrue", "foundfalse",
         --
-        "inlineordisplaymath","indisplaymath","forcedisplaymath","startforceddisplaymath","stopforceddisplaymath","startpickupmath","stoppickupmath","reqno",
+        "inlineordisplaymath", "indisplaymath", "forcedisplaymath", "startforceddisplaymath", "stopforceddisplaymath",
+        "startpickupmath", "stoppickupmath", "reqno", "forceinlinemath",
         --
         "mathortext",
         --
+        "thebox",
         "htdp",
         "unvoidbox",
         "hfilll", "vfilll",
@@ -208,39 +308,52 @@ return {
         --
         "hglue", "vglue", "hfillneg", "vfillneg", "hfilllneg", "vfilllneg",
         --
+        "hsplit",
+        --
         "ruledhss", "ruledhfil", "ruledhfill", "ruledhfilll", "ruledhfilneg", "ruledhfillneg", "normalhfillneg",  "normalhfilllneg",
         "ruledvss", "ruledvfil", "ruledvfill", "ruledvfilll", "ruledvfilneg", "ruledvfillneg", "normalvfillneg",  "normalvfilllneg",
-        "ruledhbox", "ruledvbox", "ruledvtop", "ruledvcenter", "ruledmbox",
-        "ruledhpack", "ruledvpack", "ruledtpack",
+        "ruledhbox", "ruledvbox", "ruledvtop", "ruleddbox", "ruledvcenter", "ruledmbox",
+        "ruledhpack", "ruledvpack", "ruledtpack", "ruleddpack",
+        "ruledvsplit", "ruledtsplit", "ruleddsplit",
         "ruledhskip", "ruledvskip", "ruledkern", "ruledmskip", "ruledmkern",
         "ruledhglue", "ruledvglue", "normalhglue", "normalvglue",
         "ruledpenalty",
         --
         "filledhboxb", "filledhboxr", "filledhboxg", "filledhboxc", "filledhboxm", "filledhboxy", "filledhboxk",
         --
+        "scratchstring", "scratchstringone", "scratchstringtwo", "tempstring",
         "scratchcounter", "globalscratchcounter", "privatescratchcounter",
         "scratchdimen", "globalscratchdimen", "privatescratchdimen",
         "scratchskip", "globalscratchskip", "privatescratchskip",
         "scratchmuskip", "globalscratchmuskip", "privatescratchmuskip",
         "scratchtoks", "globalscratchtoks", "privatescratchtoks",
         "scratchbox", "globalscratchbox", "privatescratchbox",
+        "scratchmacro", "scratchmacroone", "scratchmacrotwo",
+        --
+        "scratchconditiontrue", "scratchconditionfalse", "ifscratchcondition",
+        "scratchconditiononetrue", "scratchconditiononefalse", "ifscratchconditionone",
+        "scratchconditiontwotrue", "scratchconditiontwofalse", "ifscratchconditiontwo",
         --
         "globalscratchcounterone", "globalscratchcountertwo", "globalscratchcounterthree",
         --
         "groupedcommand", "groupedcommandcs",
         "triggergroupedcommand", "triggergroupedcommandcs",
-        "simplegroupedcommand", "pickupgroupedcommand",
+        "simplegroupedcommand", "simplegroupedcommandcs",
+        "pickupgroupedcommand", "pickupgroupedcommandcs",
+        "mathgroupedcommandcs",
         --
-        "normalbaselineskip", "normallineskip", "normallineskiplimit",
+        "usedbaselineskip", "usedlineskip", "usedlineskiplimit",
         --
         "availablehsize", "localhsize", "setlocalhsize", "distributedhsize", "hsizefraction",
         --
         "next", "nexttoken",
         --
         "nextbox", "dowithnextbox", "dowithnextboxcs", "dowithnextboxcontent", "dowithnextboxcontentcs", "flushnextbox",
-        "boxisempty",
+        "boxisempty", "boxtostring", "contentostring", "prerolltostring",
         --
-        "scratchwidth", "scratchheight", "scratchdepth", "scratchoffset", "scratchdistance",
+        "givenwidth", "givenheight", "givendepth", "scangivendimensions",
+        --
+        "scratchwidth", "scratchheight", "scratchdepth", "scratchoffset", "scratchdistance", "scratchtotal", "scratchitalic",
         "scratchhsize", "scratchvsize",
         "scratchxoffset", "scratchyoffset", "scratchhoffset", "scratchvoffset",
         "scratchxposition", "scratchyposition",
@@ -263,6 +376,7 @@ return {
         "doifinset", "doifnotinset",
         "doifelseinset", "doifinsetelse",
         "doifelsenextchar", "doifnextcharelse",
+        "doifelsenextcharcs", "doifnextcharcselse",
         "doifelsenextoptional", "doifnextoptionalelse",
         "doifelsenextoptionalcs", "doifnextoptionalcselse",
         "doifelsefastoptionalcheck", "doiffastoptionalcheckelse",
@@ -284,15 +398,24 @@ return {
         "doifelsecommon", "doifcommonelse", "doifcommon", "doifnotcommon",
         "doifinstring", "doifnotinstring", "doifelseinstring", "doifinstringelse",
         "doifelseassignment", "doifassignmentelse", "docheckassignment", "doifelseassignmentcs", "doifassignmentelsecs",
+        "validassignment", "novalidassignment",
         "doiftext", "doifelsetext", "doiftextelse", "doifnottext",
         --
-        "tracingall", "tracingnone", "loggingall",
+        "quitcondition", "truecondition", "falsecondition",
+        --
+        "tracingall", "tracingnone", "loggingall", "tracingcatcodes",
+        "showluatokens",
+        --
+        "aliasmacro",
         --
         "removetoks", "appendtoks", "prependtoks", "appendtotoks", "prependtotoks", "to",
         --
-        "endgraf", "endpar", "everyendpar", "reseteverypar", "finishpar", "empty", "null", "space", "quad", "enspace", "emspace", "charspace", "nbsp", "crlf",
-        "obeyspaces", "obeylines", "obeyedspace", "obeyedline", "obeyedtab", "obeyedpage",
-        "normalspace",
+        -- "everyendpar",
+        --
+        "endgraf", "endpar", "reseteverypar", "finishpar", "empty", "null", "space", "quad", "enspace", "emspace", "charspace", "nbsp", "crlf",
+        "obeyspaces", "obeylines", "obeytabs", "obeypages", "obeyedspace", "obeyedline", "obeyedtab", "obeyedpage",
+        "normalspace", "naturalspace", "controlspace", "normalspaces",
+        "ignoretabs", "ignorelines", "ignorepages", "ignoreeofs", "setcontrolspaces",
         --
         "executeifdefined",
         --
@@ -303,25 +426,30 @@ return {
         --
         "wait", "writestatus", "define", "defineexpandable", "redefine",
         --
-        "setmeasure", "setemeasure", "setgmeasure", "setxmeasure", "definemeasure", "freezemeasure", "measure", "measured",
+        "setmeasure", "setemeasure", "setgmeasure", "setxmeasure", "definemeasure", "freezemeasure",
+        "measure", "measured", "directmeasure",
+        "setquantity", "setequantity", "setgquantity", "setxquantity", "definequantity", "freezequantity",
+        "quantity", "quantitied", "directquantity",
+     -- "quantified",
         --
         "installcorenamespace",
         --
         "getvalue", "getuvalue", "setvalue", "setevalue", "setgvalue", "setxvalue", "letvalue", "letgvalue",
         "resetvalue", "undefinevalue", "ignorevalue",
         "setuvalue", "setuevalue", "setugvalue", "setuxvalue",
-        --
-        "globallet", "glet", "udef", "ugdef", "uedef", "uxdef", "checked", "unique",
+        -- glet
+        "globallet", "udef", "ugdef", "uedef", "uxdef", "checked", "unique",
         --
         "getparameters", "geteparameters", "getgparameters", "getxparameters", "forgetparameters", "copyparameters",
         --
-        "getdummyparameters", "dummyparameter", "directdummyparameter", "setdummyparameter", "letdummyparameter", "setexpandeddummyparameter",
+        "getdummyparameters", "dummyparameter", "directdummyparameter", "setdummyparameter", "letdummyparameter",
+        "setexpandeddummyparameter", "resetdummyparameter",
         "usedummystyleandcolor", "usedummystyleparameter", "usedummycolorparameter",
         --
         "processcommalist", "processcommacommand", "quitcommalist", "quitprevcommalist",
         "processaction", "processallactions", "processfirstactioninset", "processallactionsinset",
         --
-        "unexpanded", "expanded", "startexpanded", "stopexpanded", "protected", "protect", "unprotect",
+        "unexpanded", "expanded", "startexpanded", "stopexpanded", "protect", "unprotect",
         --
         "firstofoneargument",
         "firstoftwoarguments", "secondoftwoarguments",
@@ -338,12 +466,12 @@ return {
         "gobbleoneoptional", "gobbletwooptionals", "gobblethreeoptionals", "gobblefouroptionals", "gobblefiveoptionals",
         --
         "dorecurse", "doloop", "exitloop", "dostepwiserecurse", "recurselevel", "recursedepth", "dofastloopcs", "fastloopindex", "fastloopfinal", "dowith",
+        "doloopovermatch", "doloopovermatched", "doloopoverlist",
         --
         "newconstant", "setnewconstant", "setconstant", "setconstantvalue",
-        "newconditional", "settrue", "setfalse", "settruevalue", "setfalsevalue",
+        "newconditional", "settrue", "setfalse", "settruevalue", "setfalsevalue", "setconditional",
         --
-        "newmacro", "setnewmacro", "newfraction",
-        "newsignal",
+        "newmacro", "setnewmacro", "newfraction", "newsignal", "newboundary",
         --
         "dosingleempty", "dodoubleempty", "dotripleempty", "doquadrupleempty", "doquintupleempty", "dosixtupleempty", "doseventupleempty",
         "dosingleargument", "dodoubleargument", "dotripleargument", "doquadrupleargument", "doquintupleargument", "dosixtupleargument", "doseventupleargument",
@@ -356,6 +484,8 @@ return {
         --
         "getfirstcharacter", "doifelsefirstchar", "doiffirstcharelse",
         --
+        "mathclassvalue",
+        --
         "startnointerference", "stopnointerference",
         --
         "twodigits","threedigits",
@@ -367,38 +497,69 @@ return {
         "strut", "halfstrut", "quarterstrut", "depthstrut", "halflinestrut", "noheightstrut", "setstrut", "strutbox", "strutht", "strutdp", "strutwd", "struthtdp", "strutgap", "begstrut", "endstrut", "lineheight",
         "leftboundary", "rightboundary", "signalcharacter",
         --
-        "ordordspacing", "ordopspacing", "ordbinspacing", "ordrelspacing",
-        "ordopenspacing", "ordclosespacing", "ordpunctspacing", "ordinnerspacing",
+        "aligncontentleft", "aligncontentmiddle", "aligncontentright",
         --
-        "opordspacing", "opopspacing", "opbinspacing", "oprelspacing",
-        "opopenspacing", "opclosespacing", "oppunctspacing", "opinnerspacing",
+        "shiftbox", "vpackbox", "hpackbox", "vpackedbox", "hpackedbox",
         --
-        "binordspacing", "binopspacing", "binbinspacing", "binrelspacing",
-        "binopenspacing", "binclosespacing", "binpunctspacing", "bininnerspacing",
-        --
-        "relordspacing", "relopspacing", "relbinspacing", "relrelspacing",
-        "relopenspacing", "relclosespacing", "relpunctspacing", "relinnerspacing",
-        --
-        "openordspacing", "openopspacing", "openbinspacing", "openrelspacing",
-        "openopenspacing", "openclosespacing", "openpunctspacing", "openinnerspacing",
-        --
-        "closeordspacing", "closeopspacing", "closebinspacing", "closerelspacing",
-        "closeopenspacing", "closeclosespacing", "closepunctspacing", "closeinnerspacing",
-        --
-        "punctordspacing", "punctopspacing", "punctbinspacing", "punctrelspacing",
-        "punctopenspacing", "punctclosespacing", "punctpunctspacing", "punctinnerspacing",
-        --
-        "innerordspacing", "inneropspacing", "innerbinspacing", "innerrelspacing",
-        "inneropenspacing", "innerclosespacing", "innerpunctspacing", "innerinnerspacing",
+     -- "ordordspacing", "ordopspacing", "ordbinspacing", "ordrelspacing",
+     -- "ordopenspacing", "ordclosespacing", "ordpunctspacing", "ordinnerspacing",
+     -- "ordfracspacing", "ordradspacing", "ordmiddlespacing", "ordaccentspacing",
+     -- --
+     -- "opordspacing", "opopspacing", "opbinspacing", "oprelspacing",
+     -- "opopenspacing", "opclosespacing", "oppunctspacing", "opinnerspacing",
+     -- "opfracspacing", "opradspacing", "opmiddlespacing", "opaccentspacing",
+     -- --
+     -- "binordspacing", "binopspacing", "binbinspacing", "binrelspacing",
+     -- "binopenspacing", "binclosespacing", "binpunctspacing", "bininnerspacing",
+     -- "binfracspacing", "binradspacing", "binmiddlespacing", "binaccentspacing",
+     -- --
+     -- "relordspacing", "relopspacing", "relbinspacing", "relrelspacing",
+     -- "relopenspacing", "relclosespacing", "relpunctspacing", "relinnerspacing",
+     -- "relfracspacing", "relradspacing", "relmiddlespacing", "relaccentspacing",
+     -- --
+     -- "openordspacing", "openopspacing", "openbinspacing", "openrelspacing",
+     -- "openopenspacing", "openclosespacing", "openpunctspacing", "openinnerspacing",
+     -- "openfracspacing", "openradspacing", "openmiddlespacing", "openaccentspacing",
+     -- --
+     -- "closeordspacing", "closeopspacing", "closebinspacing", "closerelspacing",
+     -- "closeopenspacing", "closeclosespacing", "closepunctspacing", "closeinnerspacing",
+     -- "closefracspacing", "closeradspacing", "closemiddlespacing", "closeaccentspacing",
+     -- --
+     -- "punctordspacing", "punctopspacing", "punctbinspacing", "punctrelspacing",
+     -- "punctopenspacing", "punctclosespacing", "punctpunctspacing", "punctinnerspacing",
+     -- "punctfracspacing", "punctradspacing", "punctmiddlespacing", "punctaccentspacing",
+     -- --
+     -- "innerordspacing", "inneropspacing", "innerbinspacing", "innerrelspacing",
+     -- "inneropenspacing", "innerclosespacing", "innerpunctspacing", "innerinnerspacing",
+     -- "innerfracspacing", "innerradspacing", "innermiddlespacing", "inneraccentspacing",
+     -- --
+     -- "fracordspacing", "fracopspacing", "fracbinspacing", "fracrelspacing",
+     -- "fracopenspacing", "fracclosespacing", "fracpunctspacing", "fracinnerspacing",
+     -- "fracfracspacing", "fracradspacing", "fracmiddlespacing", "fracaccentspacing",
+     -- --
+     -- "radordspacing", "radopspacing", "radbinspacing", "radrelspacing",
+     -- "radopenspacing", "radclosespacing", "radpunctspacing", "radinnerspacing",
+     -- "radfracspacing", "radradspacing", "radmiddlespacing", "radaccentspacing",
+     -- --
+     -- "middleordspacing", "middleopspacing", "middlebinspacing", "middlerelspacing",
+     -- "middleopenspacing", "middleclosespacing", "middlepunctspacing", "middleinnerspacing",
+     -- "middlefracspacing", "middleradspacing", "middlemiddlespacing", "middleaccentspacing",
+     -- --
+     -- "accentordspacing", "accentopspacing", "accentbinspacing", "accentrelspacing",
+     -- "accentopenspacing", "accentclosespacing", "accentpunctspacing", "accentinnerspacing",
+     -- "accentfracspacing", "accentradspacing", "accentmiddlespacing", "accentaccentspacing",
         --
         "normalreqno",
         --
         "startimath", "stopimath", "normalstartimath", "normalstopimath",
         "startdmath", "stopdmath", "normalstartdmath", "normalstopdmath",
-        "normalsuperscript", "normalsubscript", "normalnosuperscript", "normalnosubscript",
-        "superscript", "subscript", "nosuperscript", "nosubscript",
+        "normalsuperscript", "normalsubscript", "normalnosuperscript", "normalnosubscript", "normalprimescript",
+        "superscript", "subscript", "nosuperscript", "nosubscript", "primescript",
+        "superprescript", "subprescript", "nosuperprescript", "nosubsprecript",
         --
-        "uncramped", "cramped", "triggermathstyle", "mathstylefont", "mathsmallstylefont", "mathstyleface", "mathsmallstyleface", "mathstylecommand", "mathpalette",
+        "uncramped", "cramped",
+        "mathstyletrigger", "triggermathstyle", "triggeredmathstyle",
+        "mathstylefont", "mathsmallstylefont", "mathstyleface", "mathsmallstyleface", "mathstylecommand", "mathpalette",
         "mathstylehbox", "mathstylevbox", "mathstylevcenter", "mathstylevcenteredhbox", "mathstylevcenteredvbox",
         "mathtext", "setmathsmalltextbox", "setmathtextbox",
         "pushmathstyle", "popmathstyle",
@@ -417,7 +578,7 @@ return {
         "ctxlua", "luacode", "lateluacode", "directluacode",
         "registerctxluafile", "ctxloadluafile",
         "luaversion", "luamajorversion", "luaminorversion",
-        "ctxluacode", "luaconditional", "luaexpanded",
+        "ctxluacode", "luaconditional", "luaexpanded", "ctxluamatch",
         "startluaparameterset", "stopluaparameterset", "luaparameterset",
         "definenamedlua",
         "obeylualines", "obeyluatokens",
@@ -428,12 +589,14 @@ return {
         "cldprocessfile", "cldloadfile", "cldloadviafile", "cldcontext", "cldcommand",
         --
         "carryoverpar",
+        "freezeparagraphproperties", "defrostparagraphproperties",
+        "setparagraphfreezing", "forgetparagraphfreezing",
+        "updateparagraphproperties", "updateparagraphpenalties", "updateparagraphdemerits", "updateparagraphshapes", "updateparagraphlines",
+        --
         "lastlinewidth",
         --
         "assumelongusagecs",
         --
-        "Umathbotaccent",
-        --
         "righttolefthbox", "lefttorighthbox", "righttoleftvbox", "lefttorightvbox", "righttoleftvtop", "lefttorightvtop",
         "rtlhbox", "ltrhbox", "rtlvbox", "ltrvbox", "rtlvtop", "ltrvtop",
         "autodirhbox", "autodirvbox", "autodirvtop",
@@ -441,8 +604,10 @@ return {
         "lefttoright", "righttoleft", "checkedlefttoright", "checkedrighttoleft",
         "synchronizelayoutdirection","synchronizedisplaydirection","synchronizeinlinedirection",
         "dirlre", "dirrle", "dirlro", "dirrlo",
+        "rtltext", "ltrtext",
         --
-        "lesshyphens", "morehyphens", "nohyphens", "dohyphens",
+        "lesshyphens", "morehyphens", "nohyphens", "dohyphens", "dohyphencollapsing", "nohyphencollapsing",
+        "compounddiscretionary",
         --
         "Ucheckedstartdisplaymath", "Ucheckedstopdisplaymath",
         --
@@ -453,10 +618,51 @@ return {
         "naturalhbox", "naturalvbox", "naturalvtop", "naturalhpack", "naturalvpack", "naturaltpack",
         "reversehbox", "reversevbox", "reversevtop", "reversehpack", "reversevpack", "reversetpack",
         --
+        "hcontainer", "vcontainer", "tcontainer",
+        --
         "frule",
         --
         "compoundhyphenpenalty",
         --
         "start", "stop",
+        --
+        "unsupportedcs",
+        --
+        "openout", "closeout", "write", "openin", "closein", "read", "readline", "readfromterminal",
+        --
+        "boxlines", "boxline", "setboxline", "copyboxline",
+        "boxlinewd","boxlineht", "boxlinedp",
+        "boxlinenw","boxlinenh", "boxlinend",
+        "boxlinels", "boxliners", "boxlinelh", "boxlinerh",
+        "boxlinelp", "boxlinerp", "boxlinein",
+        "boxrangewd", "boxrangeht", "boxrangedp",
+        --
+        "bitwiseset", "bitwiseand", "bitwiseor", "bitwisexor", "bitwisenot", "bitwisenil",
+        "ifbitwiseand", "bitwise", "bitwiseshift", "bitwiseflip",
+        -- old ... very low level
+        "textdir", "linedir", "pardir", "boxdir",
+        --
+        "prelistbox", "postlistbox", "prelistcopy", "postlistcopy", "setprelistbox", "setpostlistbox",
+        --
+        "noligaturing", "nokerning", "noexpansion", "noprotrusion",
+        "noleftkerning", "noleftligaturing", "norightkerning", "norightligaturing", "noitaliccorrection",
+         --
+        "futureletnexttoken", "defbackslashbreak", "letbackslashbreak",
+        --
+        "pushoverloadmode", "popoverloadmode", "pushrunstate", "poprunstate",
+        --
+        "suggestedalias",
+        --
+        "showboxhere",
+        --
+        "discoptioncodestring", "flagcodestring", "frozenparcodestring", "glyphoptioncodestring", "groupcodestring",
+        "hyphenationcodestring", "mathcontrolcodestring", "mathflattencodestring", "normalizecodestring",
+        "parcontextcodestring",
+        --
+        "newlocalcount", "newlocaldimen", "newlocalskip", "newlocalmuskip", "newlocaltoks", "newlocalbox",
+        "newlocalwrite", "newlocalread",
+        "setnewlocalcount", "setnewlocaldimen", "setnewlocalskip", "setnewlocalmuskip", "setnewlocaltoks", "setnewlocalbox",
+        --
+        "ifexpression"
     }
 }
