@@ -32,14 +32,11 @@ return {
   "iffontchar",
   "interactionmode",
   "interlinepenalties",
-  "lastchkdim",
-  "lastchknum",
   "lastlinefit",
   "lastnodetype",
   "marks",
   "muexpr",
   "mutoglue",
-  "numericscale",
   "numexpr",
   "pagediscards",
   "parshapedimen",
@@ -57,7 +54,6 @@ return {
   "splitdiscards",
   "splitfirstmarks",
   "topmarks",
-  "tracingalignments",
   "tracingassigns",
   "tracinggroups",
   "tracingifs",
@@ -67,165 +63,181 @@ return {
   "widowpenalties",
  },
  ["luatex"]={
-  "UUskewed",
-  "UUskewedwithdelims",
   "Uabove",
   "Uabovewithdelims",
   "Uatop",
   "Uatopwithdelims",
   "Uchar",
   "Udelcode",
-  "Udelcodenum",
+  "Udelimited",
   "Udelimiter",
   "Udelimiterover",
   "Udelimiterunder",
   "Uhextensible",
   "Uleft",
   "Umathaccent",
+  "Umathaccentbasedepth",
   "Umathaccentbaseheight",
+  "Umathaccentbottomovershoot",
+  "Umathaccentbottomshiftdown",
+  "Umathaccentextendmargin",
+  "Umathaccentsuperscriptdrop",
+  "Umathaccentsuperscriptpercent",
+  "Umathaccenttopovershoot",
+  "Umathaccenttopshiftup",
+  "Umathaccentvariant",
+  "Umathadapttoleft",
+  "Umathadapttoright",
   "Umathaxis",
-  "Umathbinbinspacing",
-  "Umathbinclosespacing",
-  "Umathbininnerspacing",
-  "Umathbinopenspacing",
-  "Umathbinopspacing",
-  "Umathbinordspacing",
-  "Umathbinpunctspacing",
-  "Umathbinrelspacing",
+  "Umathbottomaccentvariant",
   "Umathchar",
   "Umathcharclass",
   "Umathchardef",
   "Umathcharfam",
-  "Umathcharnum",
-  "Umathcharnumdef",
   "Umathcharslot",
   "Umathclass",
-  "Umathclosebinspacing",
-  "Umathcloseclosespacing",
-  "Umathcloseinnerspacing",
-  "Umathcloseopenspacing",
-  "Umathcloseopspacing",
-  "Umathcloseordspacing",
-  "Umathclosepunctspacing",
-  "Umathcloserelspacing",
   "Umathcode",
-  "Umathcodenum",
   "Umathconnectoroverlapmin",
+  "Umathdegreevariant",
+  "Umathdelimiterextendmargin",
+  "Umathdelimiterovervariant",
+  "Umathdelimiterpercent",
+  "Umathdelimitershortfall",
+  "Umathdelimiterundervariant",
+  "Umathdenominatorvariant",
+  "Umathdict",
+  "Umathdictdef",
+  "Umathdiscretionary",
+  "Umathextrasubpreshift",
+  "Umathextrasubprespace",
+  "Umathextrasubshift",
+  "Umathextrasubspace",
+  "Umathextrasuppreshift",
+  "Umathextrasupprespace",
+  "Umathextrasupshift",
+  "Umathextrasupspace",
+  "Umathflattenedaccentbasedepth",
+  "Umathflattenedaccentbaseheight",
+  "Umathflattenedaccentbottomshiftdown",
+  "Umathflattenedaccenttopshiftup",
   "Umathfractiondelsize",
   "Umathfractiondenomdown",
   "Umathfractiondenomvgap",
   "Umathfractionnumup",
   "Umathfractionnumvgap",
   "Umathfractionrule",
-  "Umathinnerbinspacing",
-  "Umathinnerclosespacing",
-  "Umathinnerinnerspacing",
-  "Umathinneropenspacing",
-  "Umathinneropspacing",
-  "Umathinnerordspacing",
-  "Umathinnerpunctspacing",
-  "Umathinnerrelspacing",
+  "Umathfractionvariant",
+  "Umathhextensiblevariant",
   "Umathlimitabovebgap",
   "Umathlimitabovekern",
   "Umathlimitabovevgap",
   "Umathlimitbelowbgap",
   "Umathlimitbelowkern",
   "Umathlimitbelowvgap",
+  "Umathlimits",
+  "Umathnoaxis",
+  "Umathnolimits",
   "Umathnolimitsubfactor",
   "Umathnolimitsupfactor",
-  "Umathopbinspacing",
-  "Umathopclosespacing",
-  "Umathopenbinspacing",
-  "Umathopenclosespacing",
-  "Umathopeninnerspacing",
-  "Umathopenopenspacing",
-  "Umathopenopspacing",
-  "Umathopenordspacing",
-  "Umathopenpunctspacing",
-  "Umathopenrelspacing",
+  "Umathnumeratorvariant",
+  "Umathopenupdepth",
+  "Umathopenupheight",
   "Umathoperatorsize",
-  "Umathopinnerspacing",
-  "Umathopopenspacing",
-  "Umathopopspacing",
-  "Umathopordspacing",
-  "Umathoppunctspacing",
-  "Umathoprelspacing",
-  "Umathordbinspacing",
-  "Umathordclosespacing",
-  "Umathordinnerspacing",
-  "Umathordopenspacing",
-  "Umathordopspacing",
-  "Umathordordspacing",
-  "Umathordpunctspacing",
-  "Umathordrelspacing",
   "Umathoverbarkern",
   "Umathoverbarrule",
   "Umathoverbarvgap",
   "Umathoverdelimiterbgap",
+  "Umathoverdelimitervariant",
   "Umathoverdelimitervgap",
-  "Umathpunctbinspacing",
-  "Umathpunctclosespacing",
-  "Umathpunctinnerspacing",
-  "Umathpunctopenspacing",
-  "Umathpunctopspacing",
-  "Umathpunctordspacing",
-  "Umathpunctpunctspacing",
-  "Umathpunctrelspacing",
+  "Umathoverlayaccentvariant",
+  "Umathoverlinevariant",
+  "Umathphantom",
+  "Umathpresubshiftdistance",
+  "Umathpresupshiftdistance",
+  "Umathprimeraise",
+  "Umathprimeraisecomposed",
+  "Umathprimeshiftdrop",
+  "Umathprimeshiftup",
+  "Umathprimespaceafter",
+  "Umathprimevariant",
+  "Umathprimewidth",
   "Umathquad",
   "Umathradicaldegreeafter",
   "Umathradicaldegreebefore",
   "Umathradicaldegreeraise",
+  "Umathradicalextensibleafter",
+  "Umathradicalextensiblebefore",
   "Umathradicalkern",
   "Umathradicalrule",
+  "Umathradicalvariant",
   "Umathradicalvgap",
-  "Umathrelbinspacing",
-  "Umathrelclosespacing",
-  "Umathrelinnerspacing",
-  "Umathrelopenspacing",
-  "Umathrelopspacing",
-  "Umathrelordspacing",
-  "Umathrelpunctspacing",
-  "Umathrelrelspacing",
+  "Umathruledepth",
+  "Umathruleheight",
+  "Umathskeweddelimitertolerance",
   "Umathskewedfractionhgap",
   "Umathskewedfractionvgap",
+  "Umathsource",
   "Umathspaceafterscript",
   "Umathspacebeforescript",
-  "Umathspacingmode",
   "Umathstackdenomdown",
   "Umathstacknumup",
+  "Umathstackvariant",
   "Umathstackvgap",
+  "Umathsubscriptvariant",
+  "Umathsubshiftdistance",
   "Umathsubshiftdown",
   "Umathsubshiftdrop",
   "Umathsubsupshiftdown",
   "Umathsubsupvgap",
   "Umathsubtopmax",
   "Umathsupbottommin",
+  "Umathsuperscriptvariant",
+  "Umathsupshiftdistance",
   "Umathsupshiftdrop",
   "Umathsupshiftup",
   "Umathsupsubbottommax",
+  "Umathtopaccentvariant",
   "Umathunderbarkern",
   "Umathunderbarrule",
   "Umathunderbarvgap",
   "Umathunderdelimiterbgap",
+  "Umathunderdelimitervariant",
   "Umathunderdelimitervgap",
+  "Umathunderlinevariant",
+  "Umathuseaxis",
+  "Umathvextensiblevariant",
+  "Umathvoid",
+  "Umathxscale",
+  "Umathyscale",
   "Umiddle",
   "Unosubprescript",
   "Unosubscript",
   "Unosuperprescript",
   "Unosuperscript",
+  "Uoperator",
   "Uover",
   "Uoverdelimiter",
   "Uoverwithdelims",
+  "Uprimescript",
   "Uradical",
   "Uright",
   "Uroot",
+  "Urooted",
+  "Ushiftedsubprescript",
+  "Ushiftedsubscript",
+  "Ushiftedsuperprescript",
+  "Ushiftedsuperscript",
   "Uskewed",
   "Uskewedwithdelims",
   "Ustack",
   "Ustartdisplaymath",
   "Ustartmath",
+  "Ustartmathmode",
   "Ustopdisplaymath",
   "Ustopmath",
+  "Ustopmathmode",
+  "Ustretched",
+  "Ustretchedwithdelims",
   "Ustyle",
   "Usubprescript",
   "Usubscript",
@@ -240,8 +252,22 @@ return {
   "afterassigned",
   "aftergrouped",
   "aliased",
+  "aligncontent",
   "alignmark",
+  "alignmentcellsource",
+  "alignmentwrapsource",
   "aligntab",
+  "allcrampedstyles",
+  "alldisplaystyles",
+  "allmainstyles",
+  "allmathstyles",
+  "allscriptscriptstyles",
+  "allscriptstyles",
+  "allsplitstyles",
+  "alltextstyles",
+  "alluncrampedstyles",
+  "allunsplitstyles",
+  "amcode",
   "atendofgroup",
   "atendofgrouped",
   "attribute",
@@ -249,48 +275,102 @@ return {
   "automaticdiscretionary",
   "automatichyphenpenalty",
   "automigrationmode",
+  "autoparagraphmode",
   "begincsname",
   "beginlocalcontrol",
+  "beginmathgroup",
+  "beginsimplegroup",
   "boundary",
+  "boxadapt",
+  "boxanchor",
+  "boxanchors",
   "boxattribute",
   "boxdirection",
+  "boxfreeze",
+  "boxgeometry",
   "boxorientation",
+  "boxrepack",
+  "boxshift",
+  "boxsource",
+  "boxtarget",
   "boxtotal",
+  "boxvadjust",
   "boxxmove",
   "boxxoffset",
   "boxymove",
   "boxyoffset",
   "catcodetable",
+  "cdef",
+  "cdefcsname",
+  "cfcode",
   "clearmarks",
+  "constant",
+  "copymathatomrule",
+  "copymathparent",
+  "copymathspacing",
   "crampeddisplaystyle",
   "crampedscriptscriptstyle",
   "crampedscriptstyle",
   "crampedtextstyle",
+  "csactive",
   "csstring",
+  "currentloopiterator",
+  "currentloopnesting",
+  "currentmarks",
+  "dbox",
   "defcsname",
+  "deferred",
+  "detokenized",
   "dimensiondef",
+  "dimexpression",
   "directlua",
+  "dpack",
+  "dsplit",
   "edefcsname",
   "efcode",
   "endlocalcontrol",
+  "endmathgroup",
+  "endsimplegroup",
   "enforced",
+  "etoks",
   "etoksapp",
   "etokspre",
+  "eufactor",
+  "everybeforepar",
+  "everymathatom",
   "everytab",
   "exceptionpenalty",
   "expand",
+  "expandactive",
   "expandafterpars",
   "expandafterspaces",
   "expandcstoken",
   "expanded",
+  "expandedafter",
+  "expandedloop",
   "expandtoken",
   "explicitdiscretionary",
   "explicithyphenpenalty",
   "firstvalidlanguage",
+  "float",
+  "floatdef",
+  "floatexpr",
+  "flushmarks",
+  "fontcharba",
+  "fontcharta",
   "fontid",
+  "fontmathcontrol",
+  "fontspecdef",
+  "fontspecid",
+  "fontspecifiedname",
   "fontspecifiedsize",
+  "fontspecscale",
+  "fontspecxscale",
+  "fontspecyscale",
+  "fonttextcontrol",
   "formatname",
   "frozen",
+  "futurecsname",
   "futuredef",
   "futureexpand",
   "futureexpandis",
@@ -299,7 +379,9 @@ return {
   "gleaders",
   "glet",
   "gletcsname",
+  "glettonothing",
   "gluespecdef",
+  "glyph",
   "glyphdatafield",
   "glyphoptions",
   "glyphscale",
@@ -310,64 +392,116 @@ return {
   "glyphtextscale",
   "glyphxoffset",
   "glyphxscale",
+  "glyphxscaled",
   "glyphyoffset",
   "glyphyscale",
+  "glyphyscaled",
   "gtoksapp",
   "gtokspre",
+  "hccode",
   "hjcode",
+  "hmcode",
+  "holdingmigrations",
   "hpack",
   "hyphenationmin",
   "hyphenationmode",
   "ifabsdim",
+  "ifabsfloat",
   "ifabsnum",
   "ifarguments",
   "ifboolean",
   "ifchkdim",
+  "ifchkdimension",
   "ifchknum",
+  "ifchknumber",
   "ifcmpdim",
   "ifcmpnum",
   "ifcondition",
   "ifcstok",
+  "ifdimexpression",
   "ifdimval",
   "ifempty",
   "ifflags",
+  "iffloat",
+  "ifhaschar",
   "ifhastok",
   "ifhastoks",
   "ifhasxtoks",
   "ifincsname",
+  "ifinsert",
   "ifmathparameter",
   "ifmathstyle",
+  "ifnumexpression",
   "ifnumval",
   "ifparameter",
   "ifparameters",
   "ifrelax",
   "iftok",
+  "ifzerodim",
+  "ifzerofloat",
+  "ifzeronum",
   "ignorearguments",
+  "ignoredepthcriterium",
   "ignorepars",
   "immediate",
   "immutable",
+  "indexofcharacter",
+  "indexofregister",
+  "inherited",
   "initcatcodetable",
+  "insertbox",
+  "insertcopy",
+  "insertdepth",
+  "insertdistance",
+  "insertheight",
   "insertheights",
-  "insertht",
+  "insertlimit",
+  "insertmaxdepth",
+  "insertmode",
+  "insertmultiplier",
+  "insertpenalty",
+  "insertprogress",
+  "insertstorage",
+  "insertstoring",
+  "insertunbox",
+  "insertuncopy",
+  "insertwidth",
   "instance",
   "integerdef",
   "lastarguments",
+  "lastatomclass",
+  "lastboundary",
+  "lastchkdim",
+  "lastchknum",
+  "lastleftclass",
+  "lastloopiterator",
   "lastnamedcs",
   "lastnodesubtype",
+  "lastpageextra",
+  "lastparcontext",
+  "lastrightclass",
   "leftmarginkern",
   "letcharcode",
   "letcsname",
   "letfrozen",
+  "letmathatomrule",
+  "letmathparent",
+  "letmathspacing",
   "letprotected",
+  "lettonothing",
+  "linebreakcriterium",
   "linedirection",
-  "linepar",
-  "listcallbackmode",
   "localbrokenpenalty",
   "localcontrol",
   "localcontrolled",
+  "localcontrolledloop",
   "localinterlinepenalty",
   "localleftbox",
+  "localleftboxbox",
+  "localmiddlebox",
+  "localmiddleboxbox",
   "localrightbox",
+  "localrightboxbox",
   "lpcode",
   "luabytecode",
   "luabytecodecall",
@@ -379,77 +513,188 @@ return {
   "luatexbanner",
   "luatexrevision",
   "luatexversion",
-  "mathdelimitersmode",
+  "mathaccent",
+  "mathatom",
+  "mathatomglue",
+  "mathatomskip",
+  "mathbackwardpenalties",
+  "mathbeginclass",
+  "mathbinary",
+  "mathcheckfencesmode",
+  "mathclose",
+  "mathdictgroup",
+  "mathdictproperties",
   "mathdirection",
+  "mathdisplaymode",
   "mathdisplayskipmode",
+  "mathdoublescriptmode",
+  "mathendclass",
   "matheqnogapstep",
-  "mathflattenmode",
-  "mathitalicsmode",
+  "mathfenced",
+  "mathfontcontrol",
+  "mathforwardpenalties",
+  "mathfraction",
+  "mathghost",
+  "mathgluemode",
+  "mathgroupingmode",
+  "mathinner",
+  "mathleftclass",
+  "mathlimitsmode",
+  "mathmainstyle",
+  "mathmiddle",
   "mathnolimitsmode",
+  "mathopen",
+  "mathoperator",
+  "mathordinary",
+  "mathoverline",
   "mathpenaltiesmode",
+  "mathpunctuation",
+  "mathradical",
+  "mathrelation",
+  "mathrightclass",
   "mathrulesfam",
   "mathrulesmode",
-  "mathrulethicknessmode",
   "mathscale",
-  "mathscriptboxmode",
-  "mathscriptcharmode",
   "mathscriptsmode",
+  "mathslackmode",
+  "mathspacingmode",
+  "mathstackstyle",
   "mathstyle",
+  "mathstylefontid",
   "mathsurroundmode",
   "mathsurroundskip",
+  "maththreshold",
+  "mathunderline",
+  "meaningasis",
+  "meaningful",
+  "meaningfull",
+  "meaningles",
+  "meaningless",
   "mugluespecdef",
   "mutable",
   "noaligned",
+  "noatomruling",
   "noboundary",
   "nohrule",
   "norelax",
   "normalizelinemode",
+  "normalizeparmode",
   "nospaces",
   "novrule",
+  "numericscale",
+  "numexpression",
   "orelse",
+  "orphanpenalties",
+  "orphanpenalty",
   "orunless",
   "outputbox",
   "overloaded",
   "overloadmode",
+  "overshoot",
+  "pageboundary",
+  "pageextragoal",
+  "pagevsize",
   "parametercount",
-  "parattr",
+  "parametermark",
+  "parattribute",
   "pardirection",
   "permanent",
+  "pettymuskip",
+  "positdef",
   "postexhyphenchar",
   "posthyphenchar",
+  "postinlinepenalty",
   "prebinoppenalty",
   "predisplaygapfactor",
   "preexhyphenchar",
   "prehyphenchar",
+  "preinlinepenalty",
   "prerelpenalty",
   "protrudechars",
   "protrusionboundary",
   "pxdimen",
+  "quitloop",
   "quitvmode",
+  "resetmathspacing",
+  "retokenized",
   "rightmarginkern",
   "rpcode",
   "savecatcodetable",
+  "scaledemwidth",
+  "scaledexheight",
+  "scaledextraspace",
+  "scaledfontdimen",
+  "scaledinterwordshrink",
+  "scaledinterwordspace",
+  "scaledinterwordstretch",
+  "scaledmathstyle",
+  "scaledslantperpoint",
   "scantextokens",
+  "semiexpand",
+  "semiexpanded",
+  "semiprotected",
+  "setdefaultmathcodes",
   "setfontid",
+  "setmathatomrule",
+  "setmathdisplaypostpenalty",
+  "setmathdisplayprepenalty",
+  "setmathignore",
+  "setmathoptions",
+  "setmathpostpenalty",
+  "setmathprepenalty",
+  "setmathspacing",
+  "shapingpenaltiesmode",
+  "shapingpenalty",
   "snapshotpar",
+  "srule",
   "supmarkmode",
   "swapcsvalues",
+  "tabsize",
   "textdirection",
   "thewithoutunit",
+  "tinymuskip",
+  "todimension",
+  "tohexadecimal",
+  "tointeger",
   "tokenized",
   "toksapp",
   "tokspre",
   "tolerant",
+  "tomathstyle",
+  "toscaled",
+  "tosparsedimension",
+  "tosparsescaled",
   "tpack",
+  "tracingadjusts",
+  "tracingalignments",
+  "tracingexpressions",
   "tracingfonts",
+  "tracingfullboxes",
+  "tracinghyphenation",
+  "tracinginserts",
+  "tracinglevels",
+  "tracinglists",
+  "tracingmarks",
   "tracingmath",
+  "tracingnodes",
+  "tracingpenalties",
+  "tsplit",
+  "uleaders",
   "undent",
+  "unexpandedloop",
+  "unhpack",
   "unletfrozen",
   "unletprotected",
+  "untraced",
+  "unvpack",
+  "variablefam",
+  "virtualhrule",
+  "virtualvrule",
   "vpack",
   "wordboundary",
   "wrapuppar",
   "xdefcsname",
+  "xtoks",
   "xtoksapp",
   "xtokspre",
  },
@@ -523,6 +768,7 @@ return {
   "pdfobjcompresslevel",
   "pdfomitcharset",
   "pdfomitcidset",
+  "pdfomitinfodict",
   "pdfoutline",
   "pdfoutput",
   "pdfpageattr",
@@ -583,6 +829,7 @@ return {
   "accent",
   "adjdemerits",
   "advance",
+  "advanceby",
   "afterassignment",
   "aftergroup",
   "atop",
@@ -627,6 +874,7 @@ return {
   "displaywidowpenalty",
   "displaywidth",
   "divide",
+  "divideby",
   "doublehyphendemerits",
   "dp",
   "dump",
@@ -662,12 +910,10 @@ return {
   "font",
   "fontdimen",
   "fontname",
-  "fontspecifiedname",
   "futurelet",
   "gdef",
   "global",
   "globaldefs",
-  "glyph",
   "halign",
   "hangafter",
   "hangindent",
@@ -732,16 +978,12 @@ return {
   "lower",
   "lowercase",
   "mark",
-  "mathaccent",
   "mathbin",
   "mathchar",
   "mathchardef",
   "mathchoice",
-  "mathclose",
   "mathcode",
-  "mathinner",
   "mathop",
-  "mathopen",
   "mathord",
   "mathpunct",
   "mathrel",
@@ -749,8 +991,6 @@ return {
   "maxdeadcycles",
   "maxdepth",
   "meaning",
-  "meaningfull",
-  "meaningless",
   "medmuskip",
   "message",
   "middle",
@@ -760,6 +1000,7 @@ return {
   "moveright",
   "mskip",
   "multiply",
+  "multiplyby",
   "muskip",
   "muskipdef",
   "newlinechar",
@@ -774,14 +1015,12 @@ return {
   "number",
   "omit",
   "or",
-  "ordlimits",
   "outer",
   "output",
   "outputpenalty",
   "over",
   "overfullrule",
   "overline",
-  "overshoot",
   "overwithdelims",
   "pagedepth",
   "pagefilllstretch",
@@ -795,6 +1034,8 @@ return {
   "parfillleftskip",
   "parfillskip",
   "parindent",
+  "parinitleftskip",
+  "parinitrightskip",
   "parshape",
   "parskip",
   "patterns",
@@ -814,7 +1055,6 @@ return {
   "righthyphenmin",
   "rightskip",
   "romannumeral",
-  "scaledfontdimen",
   "scriptfont",
   "scriptscriptfont",
   "scriptscriptstyle",
@@ -850,14 +1090,11 @@ return {
   "thickmuskip",
   "thinmuskip",
   "time",
-  "todimension",
-  "tointeger",
   "toks",
   "toksdef",
   "tolerance",
   "topmark",
   "topskip",
-  "toscaled",
   "tracingcommands",
   "tracinglostchars",
   "tracingmacros",
@@ -869,16 +1106,15 @@ return {
   "tracingstats",
   "uccode",
   "uchyph",
+  "unboundary",
   "underline",
   "unhbox",
   "unhcopy",
-  "unhpack",
   "unkern",
   "unpenalty",
   "unskip",
   "unvbox",
   "unvcopy",
-  "unvpack",
   "uppercase",
   "vadjust",
   "valign",
