@@ -6,61 +6,58 @@ if not modules then modules = { } end modules ['sort-ini'] = {
     license   = "see context related readme files"
 }
 
--- It took a while to get there, but with Fleetwood Mac's "Don't Stop"
--- playing in the background we sort of got it done.
-
---[[<p>The code here evolved from the rather old mkii approach. There
-we concatinate the key and (raw) entry into a new string. Numbers and
-special characters get some treatment so that they sort ok. In
-addition some normalization (lowercasing, accent stripping) takes
-place and again data is appended ror prepended. Eventually these
-strings are sorted using a regular string sorter. The relative order
-of character is dealt with by weighting them. It took a while to
-figure this all out but eventually it worked ok for most languages,
-given that the right datatables were provided.</p>
-
-<p>Here we do follow a similar approach but this time we don't append
-the manipulated keys and entries but create tables for each of them
-with entries being tables themselves having different properties. In
-these tables characters are represented by numbers and sorting takes
-place using these numbers. Strings are simplified using lowercasing
-as well as shape codes. Numbers are filtered and after getting an offset
-they end up at the right end of the spectrum (more clever parser will
-be added some day). There are definitely more solutions to the problem
-and it is a nice puzzle to solve.</p>
-
-<p>In the future more methods can be added, as there is practically no
-limit to what goes into the tables. For that we will provide hooks.</p>
-
-<p>Todo: decomposition with specific order of accents, this is
-relatively easy to do.</p>
-
-<p>Todo: investigate what standards and conventions there are and see
-how they map onto this mechanism. I've learned that users can come up
-with any demand so nothing here is frozen.</p>
-
-<p>Todo: I ran into the Unicode Collation document and noticed that
-there are some similarities (like the weights) but using that method
-would still demand extra code for language specifics. One option is
-to use the allkeys.txt file for the uc vectors but then we would also
-use the collapsed key (sq, code is now commented). In fact, we could
-just hook those into the replacer code that we reun beforehand.</p>
-
-<p>In the future index entries will become more clever, i.e. they will
-have language etc properties that then can be used.</p>
-]]--
+-- It took a while to get there, but with Fleetwood Mac's "Don't Stop" playing in
+-- the background we sort of got it done.
+--
+-- The code here evolved from the rather old mkii approach. There we concatinate the
+-- key and (raw) entry into a new string. Numbers and special characters get some
+-- treatment so that they sort ok. In addition some normalization (lowercasing,
+-- accent stripping) takes place and again data is appended ror prepended.
+-- Eventually these strings are sorted using a regular string sorter. The relative
+-- order of character is dealt with by weighting them. It took a while to figure
+-- this all out but eventually it worked ok for most languages, given that the right
+-- datatables were provided.
+--
+-- Here we do follow a similar approach but this time we don't append the
+-- manipulated keys and entries but create tables for each of them with entries
+-- being tables themselves having different properties. In these tables characters
+-- are represented by numbers and sorting takes place using these numbers. Strings
+-- are simplified using lowercasing as well as shape codes. Numbers are filtered and
+-- after getting an offset they end up at the right end of the spectrum (more clever
+-- parser will be added some day). There are definitely more solutions to the
+-- problem and it is a nice puzzle to solve.
+--
+-- In the future more methods can be added, as there is practically no limit to what
+-- goes into the tables. For that we will provide hooks.
+--
+-- Todo: decomposition with specific order of accents, this is relatively easy to
+-- do.
+--
+-- Todo: investigate what standards and conventions there are and see how they map
+-- onto this mechanism. I've learned that users can come up with any demand so
+-- nothing here is frozen.
+--
+-- Todo: I ran into the Unicode Collation document and noticed that there are some
+-- similarities (like the weights) but using that method would still demand extra
+-- code for language specifics. One option is to use the allkeys.txt file for the uc
+-- vectors but then we would also use the collapsed key (sq, code is now commented).
+-- In fact, we could just hook those into the replacer code that we reun beforehand.
+--
+-- In the future index entries will become more clever, i.e. they will have language
+-- etc properties that then can be used.
 
 local gsub, find, rep, sub, sort, concat, tohash, format = string.gsub, string.find, string.rep, string.sub, table.sort, table.concat, table.tohash, string.format
-local utfbyte, utfchar, utfcharacters, utfvalues = utf.byte, utf.char, utf.characters, utf.values
+local utfbyte, utfchar, utfcharacters = utf.byte, utf.char, utf.characters
 local next, type, tonumber, rawget, rawset = next, type, tonumber, rawget, rawset
-local P, Cs, R, S, lpegmatch = lpeg.P, lpeg.Cs, lpeg.R, lpeg.S, lpeg.match
+local P, Cs, R, S, lpegmatch, lpegpatterns = lpeg.P, lpeg.Cs, lpeg.R, lpeg.S, lpeg.match, lpeg.patterns
 
 local allocate          = utilities.storage.allocate
 local setmetatableindex = table.setmetatableindex
 
-local trace_tests       = false  trackers.register("sorters.tests",   function(v) trace_tests   = v end)
-local trace_methods     = false  trackers.register("sorters.methods", function(v) trace_methods = v end)
-local trace_orders      = false  trackers.register("sorters.orders",  function(v) trace_orders  = v end)
+local trace_tests       = false  trackers.register("sorters.tests",        function(v) trace_tests        = v end)
+local trace_methods     = false  trackers.register("sorters.methods",      function(v) trace_methods      = v end)
+local trace_orders      = false  trackers.register("sorters.orders",       function(v) trace_orders       = v end)
+local trace_replacements= false  trackers.register("sorters.replacements", function(v) trace_replacements = v end)
 
 local report_sorters    = logs.reporter("languages","sorters")
 
@@ -400,8 +397,10 @@ local function basic(a,b) -- trace ea and eb
         -- hashed (shared) entries
         return 0
     end
-    local ea, eb = a.split, b.split
-    local na, nb = #ea, #eb
+    local ea = a.split
+    local eb = b.split
+    local na = #ea
+    local nb = #eb
     if na == 0 and nb == 0 then
         -- simple variant (single word)
         local result = 0
@@ -413,7 +412,8 @@ local function basic(a,b) -- trace ea and eb
             end
         end
         if result == 0 then
-            local la, lb = #ea.uc, #eb.uc
+            local la = #ea.uc
+            local lb = #eb.uc
             if la > lb then
                 return 1
             elseif lb > la then
@@ -428,7 +428,8 @@ local function basic(a,b) -- trace ea and eb
         -- complex variant, used in register (multiple words)
         local result = 0
         for i=1,nb < na and nb or na do
-            local eai, ebi = ea[i], eb[i]
+            local eai = ea[i]
+            local ebi = eb[i]
             for j=1,#sequence do
                 local m = sequence[j]
                 result = basicsort(eai[m],ebi[m])
@@ -437,7 +438,8 @@ local function basic(a,b) -- trace ea and eb
                 end
             end
             if result == 0 then
-                local la, lb = #eai.uc, #ebi.uc
+                local la = #eai.uc
+                local lb = #ebi.uc
                 if la > lb then
                     return 1
                 elseif lb > la then
@@ -523,7 +525,7 @@ local function prepare() -- todo: test \Ux{hex}
     return pattern
 end
 
-function sorters.strip(str) -- todo: only letters and such
+local function strip(str) -- todo: only letters and such
     if str and str ~= "" then
         return lpegmatch(pattern or prepare(),str)
     else
@@ -531,6 +533,8 @@ function sorters.strip(str) -- todo: only letters and such
     end
 end
 
+sorters.strip = strip
+
 local function firstofsplit(entry)
     -- numbers are left padded by spaces
     local split = entry.split
@@ -553,18 +557,48 @@ sorters.firstofsplit = firstofsplit
 -- we know what combinations make sense we can optimize this
 
 function splitters.utf(str,checked) -- we could append m and u but this is cleaner, s is for tracing
-    if #replacements > 0 then
+    local nofreplacements = #replacements
+    if nofreplacements > 0 then
         -- todo make an lpeg for this
-        for k=1,#replacements do
-            local v = replacements[k]
-            local s = v[1]
-            if find(str,s) then
-                str = gsub(str,s,v[2])
+        local replacer = replacements.replacer
+        if not replacer then
+            local rep = { }
+            for i=1,nofreplacements do
+                local r = replacements[i]
+                rep[strip(r[1])] = strip(r[2])
             end
+            replacer = lpeg.utfchartabletopattern(rep)
+            replacer = Cs((replacer/rep + lpegpatterns.utf8character)^0)
+            replacements.replacer = replacer
         end
+        local rep = lpegmatch(replacer,str)
+        if rep and rep ~= str then
+            if trace_replacements then
+                report_sorters("original   : %s",str)
+                report_sorters("replacement: %s",rep)
+            end
+            str = rep
+        end
+     -- for k=1,#replacements do
+     --     local v = replacements[k]
+     --     local s = v[1]
+     --     if find(str,s) then
+     --         str = gsub(str,s,v[2])
+     --     end
+     -- end
     end
-    local m_case, z_case, p_case, m_mapping, z_mapping, p_mapping, char, byte, n = { }, { }, { }, { }, { }, { }, { }, { }, 0
-    local nm, nz, np = 0, 0, 0
+    local m_case    = { }
+    local z_case    = { }
+    local p_case    = { }
+    local m_mapping = { }
+    local z_mapping = { }
+    local p_mapping = { }
+    local char      = { }
+    local byte      = { }
+    local n         = 0
+    local nm        = 0
+    local nz        = 0
+    local np        = 0
     for sc in utfcharacters(str) do
         local b = utfbyte(sc)
         if b >= digitsoffset then
@@ -797,3 +831,16 @@ function sorters.sort(entries,cmp)
         end)
     end
 end
+
+-- helper
+
+function sorters.replacementlist(list)
+    local replacements = { }
+    for i=1,#list do
+        replacements[i] = {
+            list[i],
+            utfchar(replacementoffset+i),
+        }
+    end
+    return replacements
+end
