@@ -10,7 +10,7 @@ local lpeg, table, string = lpeg, table, string
 local P, R, V, S, C, Ct, Cs, Carg, Cc, Cg, Cf, Cp = lpeg.P, lpeg.R, lpeg.V, lpeg.S, lpeg.C, lpeg.Ct, lpeg.Cs, lpeg.Carg, lpeg.Cc, lpeg.Cg, lpeg.Cf, lpeg.Cp
 local lpegmatch, lpegpatterns = lpeg.match, lpeg.patterns
 local concat, gmatch, find = table.concat, string.gmatch, string.find
-local tostring, type, next, rawset = tostring, type, next, rawset
+local tonumber, tostring, type, next, rawset = tonumber, tostring, type, next, rawset
 local mod, div = math.mod, math.div
 
 utilities         = utilities or {}
@@ -24,13 +24,14 @@ local sortedhash        = table.sortedhash
 local sortedkeys        = table.sortedkeys
 local tohash            = table.tohash
 
-local hashes             = { }
-utilities.parsers.hashes = hashes
+local hashes            = { }
+parsers.hashes          = hashes
 -- we share some patterns
 
 local digit       = R("09")
 local space       = P(' ')
 local equal       = P("=")
+local colon       = P(":")
 local comma       = P(",")
 local lbrace      = P("{")
 local rbrace      = P("}")
@@ -54,9 +55,13 @@ local nobracket   = 1 - (lbracket + rbracket)
 
 local escape, left, right = P("\\"), P('{'), P('}')
 
+-- lpegpatterns.balanced = P {
+--     [1] = ((escape * (left+right)) + (1 - (left+right)) + V(2))^0,
+--     [2] = left * V(1) * right
+-- }
 lpegpatterns.balanced = P {
-    [1] = ((escape * (left+right)) + (1 - (left+right)) + V(2))^0,
-    [2] = left * V(1) * right
+    ((escape * (left+right)) + (1 - (left+right)) + V(2))^0,
+    left * V(1) * right
 }
 
 local nestedbraces   = P { lbrace   * (nobrace   + V(1))^0 * rbrace }
@@ -66,17 +71,20 @@ local spaces         = space^0
 local argument       = Cs((lbrace/"") * ((nobrace + nestedbraces)^0) * (rbrace/""))
 local content        = (1-endofstring)^0
 
-lpegpatterns.nestedbraces  = nestedbraces  -- no capture
-lpegpatterns.nestedparents = nestedparents -- no capture
-lpegpatterns.nested        = nestedbraces  -- no capture
-lpegpatterns.argument      = argument      -- argument after e.g. =
-lpegpatterns.content       = content       -- rest after e.g =
+lpegpatterns.nestedbraces   = nestedbraces   -- no capture
+lpegpatterns.nestedparents  = nestedparents  -- no capture
+lpegpatterns.nestedbrackets = nestedbrackets -- no capture
+lpegpatterns.nested         = nestedbraces   -- no capture
+lpegpatterns.argument       = argument       -- argument after e.g. =
+lpegpatterns.content        = content        -- rest after e.g =
 
-local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace) + C((nestedbraces + (1-comma))^0)
+local value     = lbrace * C((nobrace + nestedbraces)^0) * rbrace
+                + C((nestedbraces + (1-comma))^0)
 
 local key       = C((1-equal-comma)^1)
 local pattern_a = (space+comma)^0 * (key * equal * value + key * C(""))
 local pattern_c = (space+comma)^0 * (key * equal * value)
+local pattern_d = (space+comma)^0 * (key * (equal+colon) * value + key * C(""))
 
 local key       = C((1-space-equal-comma)^1)
 local pattern_b = spaces * comma^0 * spaces * (key * ((spaces * equal * spaces * value) + C("")))
@@ -92,10 +100,12 @@ end
 local pattern_a_s = (pattern_a/set)^1
 local pattern_b_s = (pattern_b/set)^1
 local pattern_c_s = (pattern_c/set)^1
+local pattern_d_s = (pattern_d/set)^1
 
 patterns.settings_to_hash_a = pattern_a_s
 patterns.settings_to_hash_b = pattern_b_s
 patterns.settings_to_hash_c = pattern_c_s
+patterns.settings_to_hash_d = pattern_d_s
 
 function parsers.make_settings_to_hash_pattern(set,how)
     if how == "strict" then
@@ -126,6 +136,18 @@ function parsers.settings_to_hash(str,existing)
     end
 end
 
+function parsers.settings_to_hash_colon_too(str)
+    if not str or str == "" then
+        return { }
+    elseif type(str) == "table" then
+        return str
+    else
+        hash = { }
+        lpegmatch(pattern_d_s,str)
+        return hash
+    end
+end
+
 function parsers.settings_to_hash_tolerant(str,existing)
     if not str or str == "" then
         return { }
@@ -165,7 +187,7 @@ function parsers.settings_to_hash_strict(str,existing)
 end
 
 local separator = comma * space^0
-local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
+local value     = lbrace * C((nobrace + nestedbraces)^0) * rbrace
                 + C((nestedbraces + (1-comma))^0)
 local pattern   = spaces * Ct(value*(separator*value)^0)
 
@@ -210,7 +232,7 @@ function parsers.settings_to_numbers(str)
     return str
 end
 
-local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
+local value     = lbrace * C((nobrace + nestedbraces)^0) * rbrace
                 + C((nestedbraces + nestedbrackets + nestedparents + (1-comma))^0)
 local pattern   = spaces * Ct(value*(separator*value)^0)
 
@@ -242,8 +264,13 @@ function parsers.groupedsplitat(symbol,withaction)
     if not pattern then
         local symbols   = S(symbol)
         local separator = space^0 * symbols * space^0
-        local value     = P(lbrace * C((nobrace + nestedbraces)^0) * rbrace)
-                        + C((nestedbraces + (1-(space^0*(symbols+P(-1)))))^0)
+        local value     =
+                        lbrace
+                        * C((nobrace + nestedbraces)^0)
+                     -- * rbrace
+                        * (rbrace * (#symbols + P(-1))) -- new per 2023-03-11
+                        +
+                        C((nestedbraces + (1-(space^0*(symbols+P(-1)))))^0)
         if withaction then
             local withvalue = Carg(1) * value / function(f,s) return f(s) end
             pattern = spaces * withvalue * (separator*withvalue)^0
@@ -291,7 +318,9 @@ end
 
 function parsers.hash_to_string(h,separator,yes,no,strict,omit)
     if h then
-        local t, tn, s = { }, 0, sortedkeys(h)
+        local t  = { }
+        local tn = 0
+        local s  = sortedkeys(h)
         omit = omit and tohash(omit)
         for i=1,#s do
             local key = s[i]
@@ -344,7 +373,7 @@ end
 
 local pattern = Cf(Ct("") * Cg(C((1-S(", "))^1) * S(", ")^0 * Cc(true))^1,rawset)
 
-function utilities.parsers.settings_to_set(str)
+function parsers.settings_to_set(str)
     return str and lpegmatch(pattern,str) or { }
 end
 
@@ -354,10 +383,13 @@ hashes.settings_to_set =  table.setmetatableindex(function(t,k) -- experiment, n
     return v
 end)
 
+-- as we use a next, we are not sure when the gc kicks in
+
 getmetatable(hashes.settings_to_set).__mode = "kv" -- could be an option (maybe sharing makes sense)
 
 function parsers.simple_hash_to_string(h, separator)
-    local t, tn = { }, 0
+    local t  = { }
+    local tn = 0
     for k, v in sortedhash(h) do
         if v then
             tn = tn + 1
@@ -373,13 +405,13 @@ local str      = Cs(lpegpatterns.unquoted) + C((1-whitespace-equal)^1)
 local setting  = Cf( Carg(1) * (whitespace^0 * Cg(str * whitespace^0 * (equal * whitespace^0 * str + Cc(""))))^1,rawset)
 local splitter = setting^1
 
-function utilities.parsers.options_to_hash(str,target)
+function parsers.options_to_hash(str,target)
     return str and lpegmatch(splitter,str,1,target or { }) or { }
 end
 
 local splitter = lpeg.tsplitat(" ")
 
-function utilities.parsers.options_to_array(str)
+function parsers.options_to_array(str)
     return str and lpegmatch(splitter,str) or { }
 end
 
@@ -398,7 +430,8 @@ local function repeater(n,str)
         if n == 1 then
             return unpack(s)
         else
-            local t, tn = { }, 0
+            local t  = { }
+            local tn = 0
             for i=1,n do
                 for j=1,#s do
                     tn = tn + 1
@@ -517,13 +550,15 @@ function parsers.csvsplitter(specification)
     specification   = specification and setmetatableindex(specification,defaultspecification) or defaultspecification
     local separator = specification.separator
     local quotechar = specification.quote
+    local numbers   = specification.numbers
     local separator = S(separator ~= "" and separator or ",")
     local whatever  = C((1 - separator - newline)^0)
     if quotechar and quotechar ~= "" then
         local quotedata = nil
         for chr in gmatch(quotechar,".") do
             local quotechar = P(chr)
-            local quoteword = quotechar * C((1 - quotechar)^0) * quotechar
+            local quoteitem = (1 - quotechar)^0
+            local quoteword = quotechar * (numbers and (quoteitem/tonumber) or C(quoteitem)) * quotechar
             if quotedata then
                 quotedata = quotedata + quoteword
             else
@@ -538,29 +573,29 @@ function parsers.csvsplitter(specification)
     end
 end
 
--- and this is a slightly patched version of a version posted by Philipp Gesang
-
--- local mycsvsplitter = utilities.parsers.rfc4180splitter()
-
 -- local crap = [[
 -- first,second,third,fourth
 -- "1","2","3","4"
--- "a","b","c","d"
--- "foo","bar""baz","boogie","xyzzy"
+-- "5","6","7","8"
 -- ]]
 
--- local list, names = mycsvsplitter(crap,true)   inspect(list) inspect(names)
--- local list, names = mycsvsplitter(crap)        inspect(list) inspect(names)
+-- local mycsvsplitter = parsers.csvsplitter { numbers = true }
+
+-- local list = mycsvsplitter(crap) inspect(list)
+
+-- and this is a slightly patched version of a version posted by Philipp Gesang
 
 function parsers.rfc4180splitter(specification)
     specification     = specification and setmetatableindex(specification,defaultspecification) or defaultspecification
+    local numbers     = specification.numbers
     local separator   = specification.separator --> rfc: COMMA
     local quotechar   = P(specification.quote)  -->      DQUOTE
     local dquotechar  = quotechar * quotechar   -->      2DQUOTE
                       / specification.quote
     local separator   = S(separator ~= "" and separator or ",")
+    local whatever    = (dquotechar + (1 - quotechar))^0
     local escaped     = quotechar
-                      * Cs((dquotechar + (1 - quotechar))^0)
+                      * (numbers and (whatever/tonumber) or Cs(whatever))
                       * quotechar
     local non_escaped = C((1 - quotechar - newline - separator)^1)
     local field       = escaped + non_escaped + Cc("")
@@ -580,11 +615,17 @@ function parsers.rfc4180splitter(specification)
     end
 end
 
--- utilities.parsers.stepper("1,7-",9,function(i) print(">>>",i) end)
--- utilities.parsers.stepper("1-3,7,8,9")
--- utilities.parsers.stepper("1-3,6,7",function(i) print(">>>",i) end)
--- utilities.parsers.stepper(" 1 : 3, ,7 ")
--- utilities.parsers.stepper("1:4,9:13,24:*",30)
+-- local mycsvsplitter = parsers.rfc4180splitter { numbers = true }
+--
+-- local crap = [[
+-- first,second,third,fourth
+-- "1","2","3","4"
+-- "a","b","c","d"
+-- "foo","bar""baz","boogie","xyzzy"
+-- ]]
+--
+-- local list, names = mycsvsplitter(crap,true)   inspect(list) inspect(names)
+-- local list, names = mycsvsplitter(crap)        inspect(list) inspect(names)
 
 local function ranger(first,last,n,action)
     if not first then
@@ -602,7 +643,7 @@ local function ranger(first,last,n,action)
     end
 end
 
-local cardinal    = lpegpatterns.cardinal / tonumber
+local cardinal    = (lpegpatterns.hexadecimal + lpegpatterns.cardinal) / tonumber
 local spacers     = lpegpatterns.spacer^0
 local endofstring = lpegpatterns.endofstring
 
@@ -613,14 +654,29 @@ local stepper  = spacers * ( cardinal * ( spacers * S(":-") * spacers * ( cardin
                * Carg(1) * Carg(2) / ranger * S(", ")^0 )^1 * endofstring -- we're sort of strict (could do without endofstring)
 
 function parsers.stepper(str,n,action)
+    local ts = type(str)
     if type(n) == "function" then
-        lpegmatch(stepper,str,1,false,n or print)
-    else
+        if ts == "number" then
+            n(str)
+        elseif ts == "table" then
+            for i=1,#str do
+                n(str[i])
+            end
+        else
+            lpegmatch(stepper,str,1,false,n or print)
+        end
+    elseif ts == "string" then
         lpegmatch(stepper,str,1,n,action or print)
     end
 end
 
---
+-- parsers.stepper("1,7-",9,function(i) print(">>>",i) end)
+-- parsers.stepper("1-3,7,8,9")
+-- parsers.stepper("1-3,6,7",function(i) print(">>>",i) end)
+-- parsers.stepper(" 1 : 3, ,7 ")
+-- parsers.stepper("1:4,9:13,24:*",30)
+-- parsers.stepper(1,print)
+-- parsers.stepper({1,3,4},print)
 
 local pattern_math = Cs((P("%")/"\\percent " +  P("^")           * Cc("{") * lpegpatterns.integer * Cc("}") + anything)^0)
 local pattern_text = Cs((P("%")/"\\percent " + (P("^")/"\\high") * Cc("{") * lpegpatterns.integer * Cc("}") + anything)^0)
@@ -637,14 +693,14 @@ function parsers.unittoxml(str)
     return lpegmatch(pattern,str)
 end
 
--- print(utilities.parsers.unittotex("10^-32 %"),utilities.parsers.unittoxml("10^32 %"))
+-- print(parsers.unittotex("10^-32 %"),utilities.parsers.unittoxml("10^32 %"))
 
 local cache   = { }
 local spaces  = lpegpatterns.space^0
 local dummy   = function() end
 
 setmetatableindex(cache,function(t,k)
-    local separator = P(k)
+    local separator = S(k) -- was P
     local value     = (1-separator)^0
     local pattern   = spaces * C(value) * separator^0 * Cp()
     t[k] = pattern
@@ -653,7 +709,7 @@ end)
 
 local commalistiterator = cache[","]
 
-function utilities.parsers.iterator(str,separator)
+function parsers.iterator(str,separator)
     local n = #str
     if n == 0 then
         return dummy
@@ -672,7 +728,7 @@ function utilities.parsers.iterator(str,separator)
     end
 end
 
--- for s in utilities.parsers.iterator("a b c,b,c") do
+-- for s in parsers.iterator("a b c,b,c") do
 --     print(s)
 -- end
 
@@ -704,7 +760,7 @@ local name   = C((1-S(", "))^1)
 local parser = (Carg(1) * name / initialize) * (S(", ")^1 * (Carg(1) * name / fetch))^0
 local merge  = Cf(parser,process)
 
-function utilities.parsers.mergehashes(hash,list)
+function parsers.mergehashes(hash,list)
     return lpegmatch(merge,list,1,hash)
 end
 
@@ -714,9 +770,9 @@ end
 --     cc = { epsilon = 3 },
 -- }
 --
--- inspect(utilities.parsers.mergehashes(t,"aa, bb, cc"))
+-- inspect(parsers.mergehashes(t,"aa, bb, cc"))
 
-function utilities.parsers.runtime(time)
+function parsers.runtime(time)
     if not time then
         time = os.runtime()
     end
@@ -738,7 +794,7 @@ local token   = lbrace * C((1-rbrace)^1) * rbrace + C(anything^1)
 
 local pattern = spacing * (method * spacing * apply + Carg(1)) * spacing * token
 
-function utilities.parsers.splitmethod(str,default)
+function parsers.splitmethod(str,default)
     if str then
         return lpegmatch(pattern,str,1,default or false)
     else
@@ -746,9 +802,49 @@ function utilities.parsers.splitmethod(str,default)
     end
 end
 
--- print(utilities.parsers.splitmethod(" foo -> {bar} "))
--- print(utilities.parsers.splitmethod("foo->{bar}"))
--- print(utilities.parsers.splitmethod("foo->bar"))
--- print(utilities.parsers.splitmethod("foo"))
--- print(utilities.parsers.splitmethod("{foo}"))
--- print(utilities.parsers.splitmethod())
+-- print(parsers.splitmethod(" foo -> {bar} "))
+-- print(parsers.splitmethod("foo->{bar}"))
+-- print(parsers.splitmethod("foo->bar"))
+-- print(parsers.splitmethod("foo"))
+-- print(parsers.splitmethod("{foo}"))
+-- print(parsers.splitmethod())
+
+local p_year = lpegpatterns.digit^4 / tonumber
+
+local pattern = Cf( Ct("") *
+    (
+        (              Cg(Cc("year")  * p_year)
+          *  S("-/") * Cg(Cc("month") * cardinal)
+          *  S("-/") * Cg(Cc("day")   * cardinal)
+        ) +
+        (              Cg(Cc("day")   * cardinal)
+          *  S("-/") * Cg(Cc("month") * cardinal)
+          *  S("-/") * Cg(Cc("year")  * p_year)
+        ) +
+        (              Cg(Cc("year")  * p_year)
+          *  S("-/") * Cg(Cc("month") * cardinal)
+        ) +
+        (              Cg(Cc("month")  * cardinal)
+          *  S("-/") * Cg(Cc("year")   * p_year)
+        )
+    )
+      *  (
+         P(" ")  * Cg(Cc("hour")   * cardinal)
+      *  P(":")  * Cg(Cc("min")    * cardinal)
+      * (P(":")  * Cg(Cc("sec")    * cardinal))^-1
+      + P(-1) )
+
+, rawset)
+
+lpegpatterns.splittime = pattern
+
+function parsers.totime(str)
+    return lpegmatch(pattern,str)
+end
+
+-- inspect(parsers.totime("2019-03-05"))
+-- inspect(parsers.totime("2019-03-05 12:12:12"))
+-- print(os.time(parsers.totime("2019-03-05 12:12:12")))
+-- print(os.time(parsers.totime("2019/03/05 12:12:12")))
+-- print(os.time(parsers.totime("05-03-2019 12:12:12")))
+-- print(os.time(parsers.totime("05/03/2019 12:12:12")))
